"""
(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

import falcon

from freezer_api.api.common import resource
from freezer_api.common import exceptions as freezer_api_exc
from freezer_api import policy


class ClientsCollectionResource(resource.BaseResource):
    """
    Handler for endpoint: /v2/{project_id}/clients
    """
    def __init__(self, storage_driver):
        self.db = storage_driver

    @policy.enforce('clients:get_all')
    def on_get(self, req, resp, project_id):
        # GET /v2/{project_id}/clients(?limit,offset)     Lists clients
        user_id = req.get_header('X-User-ID')
        offset = req.get_param_as_int('offset') or 0
        limit = req.get_param_as_int('limit') or 10
        search = self.json_body(req)
        obj_list = self.db.get_client(project_id=project_id,
                                      user_id=user_id,
                                      offset=offset,
                                      limit=limit,
                                      search=search)
        resp.body = {'clients': obj_list}

    @policy.enforce('clients:create')
    def on_post(self, req, resp, project_id):
        # POST /v2/{project_id}/clients    Creates client entry
        doc = self.json_body(req)
        if not doc:
            raise freezer_api_exc.BadDataFormat(
                message='Missing request body')
        user_id = req.get_header('X-User-ID')
        client_id = self.db.add_client(
            project_id=project_id, user_id=user_id, doc=doc)
        resp.status = falcon.HTTP_201
        resp.body = {'client_id': client_id}


class ClientsResource(resource.BaseResource):
    """
    Handler for endpoint: /v2/{project_id}/clients/{client_id}
    """
    def __init__(self, storage_driver):
        self.db = storage_driver

    @policy.enforce('clients:get')
    def on_get(self, req, resp, project_id, client_id):
        # GET /v1/clients(?limit,offset)
        # search in body
        user_id = req.get_header('X-User-ID') or ''
        obj = self.db.get_client(project_id=project_id,
                                 user_id=user_id,
                                 client_id=client_id)
        if obj:
            resp.body = obj[0]
        else:
            resp.status = falcon.HTTP_404

    @policy.enforce('clients:delete')
    def on_delete(self, req, resp, project_id, client_id):
        # DELETE /v2/{project_id}/clients/{client_id}
        # Deletes the specified client
        user_id = req.get_header('X-User-ID')
        self.db.delete_client(project_id=project_id,
                              user_id=user_id,
                              client_id=client_id)
        resp.body = {'client_id': client_id}
        resp.status = falcon.HTTP_204



"""
Copyright 2015 Hewlett-Packard

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

fqdn_regex = "([a-zA-Z\\d]|[a-zA-Z\\d][a-zA-Z\\d\\-]{0,61}[a-zA-Z\\d])" \
             "(\\.([a-zA-Z\\d]|[a-zA-Z\\d][a-zA-Z\\d\\-]{0,61}[a-zA-Z\\d]))*"
# Allow alphanumeric, underscore and minus
project_regex = r"(?:([\w\-]+)\_)?"
client_id_regex = r"^{project}{host}$".format(project=project_regex,
                                              host=fqdn_regex)

freezer_action_properties = {
    "action": {
        "id": "action",
        "pattern": r"^[\w-]+$",
        "type": "string"
    },
    "mode": {
        "id": "mode",
        "pattern": r"^[\w-]+$",
        "type": "string"
    },
    "path_to_backup": {
        "id": "path_to_backup",
        "type": "string"
    },
    "backup_name": {
        "id": "backup_name",
        "type": "string"
    },
    "container": {
        "id": "container",
        "type": "string"
    },
    "restore_abs_path": {
        "id": "restore_abs_path",
        "type": "string"
    },
}

schedule_properties = {
    "time_created": {
        "id": "time_created",
        "type": "integer"
    },
    "time_started": {
        "id": "time_started",
        "type": "integer"
    },
    "time_ended": {
        "id": "time_ended",
        "type": "integer"
    },
    "event": {
        "id": "event",
        "type": "string",
        "enum": ["", "stop", "start", "abort", "remove"]
    },
    "status": {
        "id": "status",
        "type": "string",
        "enum": ["", "completed", "stop", "scheduled",
                 "running", "aborting", "removed"]
    },
    "result": {
        "id": "result",
        "type": "string",
        "enum": ["", "success", "fail", "aborted"]
    },
    "schedule_date": {
        "id": "schedule_date",
        "type": "string",
        "pattern": r"^(-?(?:[1-9][0-9]*)?[0-9]{4})-(1[0-2]|0[1-9])"
                   r"-(3[01]|0[1-9]|[12][0-9])T(2[0-3]|[01][0-9])"
                   r":([0-5][0-9]):([0-5][0-9])(\.[0-9]+)?(Z|[+-]"
                   r"(?:2[0-3]|[01][0-9]):[0-5][0-9])?$"
    },
    "schedule_interval": {
        "id": "schedule_interval",
        "type": "string",
        "pattern": r"^(continuous|(\d+ +(weeks|days|"
                   r"hours|minutes|seconds)))$"
    },
    "schedule_start_date": {
        "id": "schedule_start_date",
        "type": "string",
        "pattern": r"^(-?(?:[1-9][0-9]*)?[0-9]{4})-(1[0-2]|0[1-9])"
                   r"-(3[01]|0[1-9]|[12][0-9])T(2[0-3]|[01][0-9]):"
                   r"([0-5][0-9]):([0-5][0-9])(\.[0-9]+)?(Z|[+-]"
                   r"(?:2[0-3]|[01][0-9]):[0-5][0-9])?$"
    },
    "schedule_end_date": {
        "id": "schedule_end_date",
        "type": "string",
        "pattern": r"^(-?(?:[1-9][0-9]*)?[0-9]{4})-(1[0-2]|0[1-9])"
                   r"-(3[01]|0[1-9]|[12][0-9])T(2[0-3]|[01][0-9])"
                   r":([0-5][0-9]):([0-5][0-9])(\.[0-9]+)?(Z|[+-]"
                   r"(?:2[0-3]|[01][0-9]):[0-5][0-9])?$"
    },
    "schedule_year": {
        "id": "schedule_year",
        "type": "string",
        "pattern": r"^\d{4}$"
    },
    "schedule_month": {
        "id": "schedule_month",
        "type": "string"
    },
    "schedule_day": {
        "id": "schedule_day",
        "type": "string"
    },
    "schedule_week": {
        "id": "schedule_week",
        "type": "string"
    },
    "schedule_day_of_week": {
        "id": "schedule_day_of_week",
        "type": "string"
    },
    "schedule_hour": {
        "id": "schedule_hour",
        "type": "string"
    },
    "schedule_minute": {
        "id": "schedule_minute",
        "type": "string"
    },
    "schedule_second": {
        "id": "schedule_second",
        "type": "string"
    },
    "current_pid": {
        "id": "current_pid",
        "type": "integer"
    }
}

job_schema = {
    "id": "/",
    "type": "object",
    "definitions": {
        "freezer_action": {
            "properties": freezer_action_properties,
            "additionalProperties": True
        },
        "job_action": {
            "properties": {
                "freezer_action": {
                    "$ref": "#/definitions/freezer_action"
                },
                "max_retries": {
                    "type": "integer"
                },
                "max_retries_interval": {
                    "type": "integer"
                },
                "mandatory": {
                    "type": "boolean"
                }
            },
            "additionalProperties": True
        },
        "job_action_list": {
            "items": {
                "$ref": "#/definitions/job_action"
            }
        }
    },
    "properties": {
        "job_actions": {
            "$ref": "#/definitions/job_action_list"
        },
        "job_schedule": {
            "id": "job_schedule",
            "type": "object",
            "properties": schedule_properties,
            "additionalProperties": False,
        },
        "job_id": {
            "id": "job_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        },
        "client_id": {
            "id": "client_id",
            "pattern": client_id_regex,
            "type": "string"
        },
        "session_id": {
            "id": "session_id",
            "pattern": r"^[\w-]*$",
            "type": "string"
        },
        "session_tag": {
            "id": "session_tag",
            "type": "integer"
        },
        "session_name": {
            "id": "session_name",
            "type": "string"
        },
        "user_id": {
            "id": "user_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        },
        "project_id": {
            "id": "project_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        },
        "description": {
            "id": "description",
            "type": "string"
        },
        "_version": {
            "id": "_version",
            "type": "integer"
        },
        "action_defaults": {
            "$ref": "#/definitions/freezer_action"
        }
    },
    "additionalProperties": False,
    "required": [
        "job_actions",
        "job_schedule",
        "job_id",
        "client_id",
        "user_id"
    ]
}

job_patch_schema = {
    "id": "/",
    "type": "object",
    "definitions": {
        "freezer_action": {
            "properties": freezer_action_properties,
            "additionalProperties": True
        },
        "job_action": {
            "properties": {
                "freezer_action": {
                    "$ref": "#/definitions/freezer_action"
                },
                "max_retries": {
                    "type": "integer"
                },
                "max_retries_interval": {
                    "type": "integer"
                },
                "mandatory": {
                    "type": "boolean"
                }
            },
            "additionalProperties": True
        },
        "job_action_list": {
            "items": {
                "$ref": "#/definitions/job_action"
            }
        }
    },
    "properties": {
        "job_actions": {
            "$ref": "#/definitions/job_action_list"
        },
        "job_schedule": {
            "id": "job_schedule",
            "type": "object",
            "properties": schedule_properties,
            "additionalProperties": False,
        },
        "job_id": {
            "id": "job_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        },
        "client_id": {
            "id": "client_id",
            "pattern": client_id_regex,
            "type": "string"
        },
        "session_id": {
            "id": "session_id",
            "pattern": r"^[\w-]*$",
            "type": "string"
        },
        "session_tag": {
            "id": "session_tag",
            "type": "integer"
        },
        "session_name": {
            "id": "session_name",
            "type": "string"
        },
        "user_id": {
            "id": "user_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        },
        "project_id": {
            "id": "project_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        },
        "description": {
            "id": "description",
            "type": "string"
        },
        "_version": {
            "id": "_version",
            "type": "integer"
        },
        "action_defaults": {
            "$ref": "#/definitions/freezer_action"
        }
    },
    "additionalProperties": False
}


additional_action_properties = {
    "action_id": {
        "id": "action_id",
        "pattern": r"^[\w-]+$",
        "type": "string"
    },
    "user_id": {
        "id": "user_id",
        "pattern": r"^[\w-]+$",
        "type": "string"
    },
    "project_id": {
        "id": "project_id",
        "pattern": r"^[\w-]+$",
        "type": "string"
    }
}


tmp_prop = freezer_action_properties.items()
tmp_add_prop = additional_action_properties.items()
joined_properties = {}
joined_properties.update(tmp_prop)
joined_properties.update(tmp_add_prop)


action_schema = {
    "id": "/",
    "type": "object",
    "properties": joined_properties,
    "additionalProperties": True,
    "required": [
        "action_id",
        "user_id",
        "freezer_action"
    ]
}

action_patch_schema = {
    "id": "/",
    "type": "object",
    "properties": joined_properties,
    "additionalProperties": True
}

session_schema = {
    "id": "/",
    "type": "object",
    "properties": {
        "schedule": {
            "id": "schedule",
            "type": "object",
            "properties": schedule_properties,
            "additionalProperties": False,
        },
        "session_id": {
            "id": "session_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        },
        "user_id": {
            "id": "user_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        },
        "project_id": {
            "id": "project_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        },
        "session_tag": {
            "id": "session_tag",
            "type": "integer"
        },
        "time_started": {
            "id": "time_started",
            "type": "integer"
        },
        "time_ended": {
            "id": "time_ended",
            "type": "integer"
        },
    },
    "additionalProperties": True,
    "required": [
        "session_id",
        "session_tag",
        "user_id",
        "schedule"
    ]
}

session_patch_schema = {
    "id": "/",
    "type": "object",
    "properties": {
        "schedule": {
            "id": "schedule",
            "type": "object",
            "properties": schedule_properties,
            "additionalProperties": False,
        },
        "session_id": {
            "id": "session_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        },
        "user_id": {
            "id": "user_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        },
        "project_id": {
            "id": "project_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        },
        "session_tag": {
            "id": "session_tag",
            "type": "integer"
        },
        "time_started": {
            "id": "time_started",
            "type": "integer"
        },
        "time_ended": {
            "id": "time_ended",
            "type": "integer"
        },
    },
    "additionalProperties": True
}

client_info = {
    "client_id": {
        "id": "client_id",
        "pattern": client_id_regex,
        "type": "string"
    },
    "hostname": {
        "id": "hostname",
        "pattern": fqdn_regex,
        "type": "string"
    },
    "description": {
        "id": "description",
        "type": "string"
    },
    "uuid": {
        "id": "uuid",
        "type": "string"
    }
}

client_schema = {
    "id": "/",
    "type": "object",
    "properties": {
        "client": {
            "id": "client",
            "type": "object",
            "properties": client_info,
            "additionalProperties": True,
            "required": [
                "client_id"
            ]
        },
        "user_id": {
            "id": "user_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        },
        "project_id": {
            "id": "project_id",
            "pattern": r"^[\w-]+$",
            "type": "string"
        }
    },
    "additionalProperties": True,
    "required": [
        "client",
        "user_id"
    ]
}



"""
Copyright 2015 Hewlett-Packard
(c) Copyright 2016 Hewlett Packard Enterprise Development Company LP

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""


clients_mapping = {
    "properties": {
        "client": {
            "properties": {
                "client_id": {
                    "index": "not_analyzed",
                    "type": "string"
                },
                "config_id": {
                    "index": "not_analyzed",
                    "type": "string"
                },
                "description": {
                    "type": "string"
                },
                "hostname": {
                    "type": "string"
                }
            }
        },
        "user_id": {
            "index": "not_analyzed",
            "type": "string"
        },
        "project_id": {
            "index": "not_analyzed",
            "type": "string"
        },
        "uuid": {
            "index": "not_analyzed",
            "type": "string"
        }
    }
}

backups_mapping = {
    "properties": {
        "backup_id": {
            "index": "not_analyzed",
            "type": "string"
        },
        "backup_metadata": {
            "properties": {
                "action": {
                    "type": "string"
                },
                "always_level": {
                    "type": "boolean"
                },
                "backup_media": {
                    "type": "string"
                },
                "backup_name": {
                    "index": "not_analyzed",
                    "type": "string"
                },
                "backup_session": {
                    "type": "long"
                },
                "backup_size_compressed": {
                    "type": "long"
                },
                "backup_size_uncompressed": {
                    "type": "long"
                },
                "broken_links": {
                    "index": "not_analyzed",
                    "type": "string"
                },
                "cli": {
                    "type": "string"
                },
                "client_os": {
                    "type": "string"
                },
                "client_version": {
                    "type": "string"
                },
                "compression": {
                    "type": "string"
                },
                "log_file": {
                    "type": "string"
                },
                "proxy": {
                    "type": "string"
                },
                "ssh_key": {
                    "type": "string"
                },
                "ssh_username": {
                    "type": "string"
                },
                "ssh_host": {
                    "type": "string"
                },
                "ssh_port": {
                    "type": "long"
                },
                "storage": {
                    "type": "string"
                },
                "compression_alg": {
                    "type": "string"
                },
                "container": {
                    "index": "not_analyzed",
                    "type": "string"
                },
                "container_segments": {
                    "type": "string"
                },
                "curr_backup_level": {
                    "type": "string"
                },
                "current_level": {
                    "type": "string"
                },
                "dry_run": {
                    "type": "boolean"
                },
                "encrypted": {
                    "type": "boolean"
                },
                "excluded_files": {
                    "type": "string"
                },
                "fs_real_path": {
                    "type": "string"
                },
                "host_name": {
                    "index": "not_analyzed",
                    "type": "string"
                },
                "hostname": {
                    "type": "string"
                },
                "level": {
                    "type": "long"
                },
                "max_level": {
                    "type": "long"
                },
                "meta_data_file": {
                    "type": "string"
                },
                "mode": {
                    "type": "string"
                },
                "no_incremental": {
                    "type": "boolean"
                },
                "overwrite": {
                    "type": "boolean"
                },
                "path_to_backup": {
                    "type": "string"
                },
                "time_stamp": {
                    "type": "string"
                },
                "timestamp": {
                    "type": "long"
                },
                "total_backup_session_size": {
                    "type": "long"
                },
                "total_broken_links": {
                    "type": "long"
                },
                "total_directories": {
                    "type": "long"
                },
                "total_fs_files": {
                    "type": "long"
                },
                "version": {
                    "type": "string"
                },
                "vol_snap_path": {
                    "type": "string"
                },
                "os_auth_version": {
                    "type": "string"
                },
                "consistency_checksum": {
                    "type": "string"
                }
            }
        },
        "user_id": {
            "index": "not_analyzed",
            "type": "string"
        },
        "user_name": {
            "type": "string"
        },
        "client_id": {
            "index": "not_analyzed",
            "type": "string"
        },
        "job_id": {
            "index": "not_analyzed",
            "type": "string"
        },
        "project_id": {
            "index": "not_analyzed",
            "type": "string"
        }
    }
}

jobs_mapping = {
    "properties": {
        "client_id": {
            "index": "not_analyzed",
            "type": "string"
        },
        "description": {
            "type": "string"
        },
        "job_actions": {
            "properties": {
                "action_id": {
                    "type": "string"
                },
                "user_id": {
                    "index": "not_analyzed",
                    "type": "string"
                },
                "project_id": {
                    "index": "not_analyzed",
                    "type": "string"
                },
                "freezer_action": {
                    "properties": {
                        "action": {
                            "type": "string"
                        },
                        "backup_name": {
                            "type": "string"
                        },
                        "container": {
                            "type": "string"
                        },
                        "hostname": {
                            "type": "string"
                        },
                        "snapshot": {
                            "type": "boolean"
                        },
                        "storage": {
                            "type": "string"
                        },
                        "dry_run": {
                            "type": "boolean"
                        },
                        "log_file": {
                            "type": "string"
                        },
                        "lvm_auto_snap": {
                            "type": "string"
                        },
                        "lvm_dirmount": {
                            "type": "string"
                        },
                        "lvm_snapname": {
                            "type": "string"
                        },
                        "lvm_snapsize": {
                            "type": "string"
                        },
                        "max_level": {
                            "type": "long"
                        },
                        "max_priority": {
                            "type": "boolean"
                        },
                        "max_segment_size": {
                            "type": "long"
                        },
                        "mode": {
                            "type": "string"
                        },
                        "mysql_conf": {
                            "type": "string"
                        },
                        "path_to_backup": {
                            "type": "string"
                        },
                        "remove_older_than": {
                            "type": "long"
                        },
                        "remove_older_then": {
                            "type": "long"
                        },
                        "restore_abs_path": {
                            "type": "string"
                        },
                        "restore_from_host": {
                            "type": "string"
                        },
                        "ssh_host": {
                            "type": "string"
                        },
                        "ssh_port": {
                            "type": "long"
                        },
                        "ssh_key": {
                            "index": "not_analyzed",
                            "type": "string"
                        },
                        "ssh_username": {
                            "type": "string"
                        },
                        "proxy": {
                            "type": "string"
                        },
                        "no_incremental": {
                            "type": "boolean"
                        },
                        "overwrite": {
                            "type": "boolean"
                        }
                    }
                },
                "mandatory": {
                    "type": "boolean"
                },
                "max_retries": {
                    "type": "long"
                },
                "max_retries_interval": {
                    "type": "long"
                }
            }
        },
        "job_event": {
            "type": "string"
        },
        "job_id": {
            "index": "not_analyzed",
            "type": "string"
        },
        "job_schedule": {
            "properties": {
                "event": {
                    "type": "string"
                },
                "result": {
                    "type": "string"
                },
                "schedule_day_of_week": {
                    "type": "string"
                },
                "schedule_hour": {
                    "type": "string"
                },
                "schedule_interval": {
                    "type": "string"
                },
                "schedule_minute": {
                    "type": "string"
                },
                "schedule_start_date": {
                    "format": "dateOptionalTime",
                    "type": "date"
                },
                "status": {
                    "type": "string"
                },
                "time_created": {
                    "type": "long"
                },
                "time_ended": {
                    "type": "long"
                },
                "time_started": {
                    "type": "long"
                },
                "current_pid": {
                    "type": "long"
                }
            }
        },
        "session_id": {
            "type": "string",
            "index": "not_analyzed"
        },
        "session_tag": {
            "type": "long"
        },
        "user_id": {
            "index": "not_analyzed",
            "type": "string"
        },
        "project_id": {
            "index": "not_analyzed",
            "type": "string"
        }
    }
}


def get_mappings():
    return {
        "jobs": jobs_mapping,
        "backups": backups_mapping,
        "clients": clients_mapping
    }



"""Freezer swift.py related tests

(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

import unittest
from unittest.mock import patch

from freezer_api.common import exceptions
from freezer_api.common import utils
from freezer_api.tests.unit import common

DATA_backup_metadata = {
    "container": "freezer_container",
    "hostname": "alpha",
    "backup_name": "important_data_backup",
    "time_stamp": 12341234,
    "curr_backup_level": 0,
    "backup_session": 12341234,
    "max_level": 5,
    "mode": "fs",
    "fs_real_path": "/blabla",
    "vol_snap_path": "/blablasnap",
    "total_broken_links": 0,
    "total_fs_files": 11,
    "total_directories": 2,
    "backup_size_uncompressed": 12112342,
    "backup_size_compressed": 1214322,
    "total_backup_session_size": 1212,
    "compression_alg": "None",
    "encrypted": "false",
    "client_os": "linux",
    "broken_links": ["link_01", "link_02"],
    "excluded_files": ["excluded_file_01", "excluded_file_02"],
    'cli': 'whatever'
}

DATA_user_id = 'AUx6F07NwlhuOVELWtHx'

DATA_user_name = 'gegrex55NPlwlhuOVELWtHv'

DATA_backup_id = 'd78f820089d64b7b9096d3133ca3162d'

DATA_wrapped_backup_metadata = {
    'user_id': DATA_user_id,
    'user_name': DATA_user_name,
    'backup_id': DATA_backup_id,
    'backup_medatada': {
        "container": "freezer_container",
        "hostname": "alpha",
        "backup_name": "important_data_backup",
        "time_stamp": 12341234,
        "curr_backup_level": 0,
        "backup_session": 12341234,
        "max_level": 5,
        "mode": "fs",
        "fs_real_path": "/blabla",
        "vol_snap_path": "/blablasnap",
        "total_broken_links": 0,
        "total_fs_files": 11,
        "total_directories": 2,
        "backup_size_uncompressed": 12112342,
        "backup_size_compressed": 1214322,
        "total_backup_session_size": 1212,
        "compression_alg": "None",
        "encrypted": "false",
        "client_os": "linux",
        "broken_links": ["link_01", "link_02"],
        "excluded_files": ["excluded_file_01", "excluded_file_02"],
        'cli': 'whatever'
    }
}


class TestBackupMetadataDoc(unittest.TestCase):
    def setUp(self):
        self.backup_metadata = utils.BackupMetadataDoc(
            user_id=DATA_user_id,
            user_name=DATA_user_name,
            data=DATA_backup_metadata)

    def test_is_valid_return_True_when_valid(self):
        self.assertTrue(self.backup_metadata.is_valid())

    def test_is_valid_returns_False_when_user_id_empty(self):
        self.backup_metadata.user_id = ''
        self.assertFalse(self.backup_metadata.is_valid())


class TestJobDoc(unittest.TestCase):
    def test_validate_ok_when_data_ok(self):
        job_doc = common.get_fake_job_0()
        self.assertIsNone(utils.JobDoc.validate(job_doc))

    def test_validate_raises_BadDataFormat_when_doc_has_no_jobid(self):
        job_doc = common.get_fake_job_0()
        job_doc.pop('job_id')
        self.assertRaises(exceptions.BadDataFormat, utils.JobDoc.validate,
                          job_doc)

    def test_validate_raises_BadDataFormat_when_doc_has_no_userid(self):
        job_doc = common.get_fake_job_0()
        job_doc.pop('job_id')
        self.assertRaises(exceptions.BadDataFormat, utils.JobDoc.validate,
                          job_doc)

    def test_validate_raises_BadDataFormat_when_doc_has_invalid_state(self):
        job_doc = common.get_fake_job_0()
        job_doc['job_schedule']['status'] = 'not_valid'
        self.assertRaises(exceptions.BadDataFormat, utils.JobDoc.validate,
                          job_doc)

    def test_validate_patch_raises_when_doc_has_invalid_state(self):
        job_doc = common.get_fake_job_0()
        job_doc['job_schedule']['status'] = 'value_no_allowed'
        self.assertRaises(exceptions.BadDataFormat,
                          utils.JobDoc.validate_patch, job_doc)

    def test_createpatch_pops_jobid_and_userid(self):
        job_doc = common.get_fake_job_0()
        res_doc = utils.JobDoc.create_patch(job_doc)
        self.assertFalse('job_id' in res_doc)
        self.assertFalse('user_id' in res_doc)

    def test_createpatch_raises_BadDataFormat_when_patch_has_invalid_state(
            self):
        job_doc = common.get_fake_job_0()
        job_doc['job_schedule']['status'] = 'value_no_allowed'
        self.assertRaises(exceptions.BadDataFormat, utils.JobDoc.create_patch,
                          job_doc)

    def test_createpatch_raises_BadDataFormat_when_patch_has_invalid_event(
            self):
        job_doc = common.get_fake_job_0()
        job_doc['job_schedule']['event'] = 'value_no_allowed'
        self.assertRaises(exceptions.BadDataFormat, utils.JobDoc.create_patch,
                          job_doc)

    @patch('freezer_api.common.utils.uuid')
    @patch('freezer_api.common.utils.time')
    def test_create_inserts_correct_uuid_timecreated_status(self, mock_time,
                                                            mock_uuid):
        mock_time.time.return_value = 1433611337
        mock_uuid.uuid4.return_value = mock_uuid
        mock_uuid.hex = 'hotforteacher'
        job_doc = common.get_fake_job_0()
        job_doc['job_schedule']['status'] = 'stop'
        res_doc = utils.JobDoc.create(job_doc, 'dude')
        self.assertEqual(1433611337, res_doc['job_schedule']['time_created'])
        self.assertEqual(-1, res_doc['job_schedule']['time_started'])
        self.assertEqual(-1, res_doc['job_schedule']['time_ended'])
        self.assertEqual('stop', res_doc['job_schedule']['status'])
        self.assertEqual('dude', res_doc['user_id'])
        self.assertEqual('hotforteacher', res_doc['job_id'])

    @patch('freezer_api.common.utils.uuid')
    @patch('freezer_api.common.utils.time')
    def test_create_raises_BadDataFormat_when_isvalid_fails(self, mock_time,
                                                            mock_uuid):
        mock_time.time.return_value = 1433611337
        mock_uuid.uuid4.return_value = mock_uuid
        mock_uuid.hex = 'hotforteacher'
        job_doc = common.get_fake_job_0()
        job_doc['job_schedule']['event'] = 'not_valid'
        self.assertRaises(exceptions.BadDataFormat, utils.JobDoc.create,
                          job_doc, 'dude')


class TestActionDoc(unittest.TestCase):
    def test_validate_ok_when_data_ok(self):
        action_doc = common.get_fake_action_0()
        self.assertIsNone(utils.ActionDoc.validate(action_doc))

    def test_validate_raises_BadDataFormat_when_doc_has_no_actionid(self):
        action_doc = common.get_fake_action_0()
        action_doc.pop('action_id')
        self.assertRaises(exceptions.BadDataFormat, utils.ActionDoc.validate,
                          action_doc)

    def test_validate_raises_BadDataFormat_when_doc_has_no_userid(self):
        action_doc = common.get_fake_action_0()
        action_doc.pop('user_id')
        self.assertRaises(exceptions.BadDataFormat, utils.ActionDoc.validate,
                          action_doc)

    def test_validate_raises_BadDataFormat_when_doc_has_invalid_field(self):
        action_doc = common.get_fake_action_0()
        action_doc['action_id'] = 44
        self.assertRaises(exceptions.BadDataFormat, utils.ActionDoc.validate,
                          action_doc)

    def test_validate_patch_raises_when_doc_has_invalid_field(self):
        action_doc = common.get_fake_action_0()
        action_doc['action_id'] = 44
        self.assertRaises(exceptions.BadDataFormat,
                          utils.ActionDoc.validate_patch, action_doc)

    def test_createpatch_pops_actionid_and_userid(self):
        action_doc = common.get_fake_action_0()
        res_doc = utils.ActionDoc.create_patch(action_doc)
        self.assertFalse('action_id' in res_doc)
        self.assertFalse('user_id' in res_doc)

    def test_createpatch_raises_BadDataFormat_when_patch_has_invalid_field(
            self):
        action_doc = common.get_fake_action_0()
        action_doc['action'] = 44
        self.assertRaises(exceptions.BadDataFormat,
                          utils.ActionDoc.create_patch, action_doc)

    @patch('freezer_api.common.utils.uuid')
    def test_create_inserts_correct_uuid(self, mock_uuid):
        mock_uuid.uuid4.return_value = mock_uuid
        mock_uuid.hex = 'hotforteacher'
        action_doc = common.get_fake_action_0()
        action_doc.pop('action_id')
        res_doc = utils.ActionDoc.create(action_doc, 'dude')
        self.assertEqual('dude', res_doc['user_id'])
        self.assertEqual('hotforteacher', res_doc['action_id'])

    @patch('freezer_api.common.utils.uuid')
    def test_create_raises_BadDataFormat_when_isvalid_fails(self, mock_uuid):
        mock_uuid.uuid4.return_value = mock_uuid
        mock_uuid.hex = 'hotforteacher'
        action_doc = common.get_fake_action_0()
        action_doc['action'] = 44
        self.assertRaises(exceptions.BadDataFormat, utils.ActionDoc.create,
                          action_doc, 'dude')


class TestSessionDoc(unittest.TestCase):
    def test_validate_ok_when_data_ok(self):
        session_doc = common.get_fake_session_0()
        self.assertIsNone(utils.SessionDoc.validate(session_doc))

    def test_validate_raises_BadDataFormat_when_doc_has_no_sessionid(self):
        session_doc = common.get_fake_session_0()
        session_doc.pop('session_id')
        self.assertRaises(exceptions.BadDataFormat, utils.SessionDoc.validate,
                          session_doc)

    def test_validate_raises_BadDataFormat_when_doc_has_no_userid(self):
        session_doc = common.get_fake_session_0()
        session_doc.pop('user_id')
        self.assertRaises(exceptions.BadDataFormat, utils.SessionDoc.validate,
                          session_doc)

    def test_validate_raises_BadDataFormat_when_doc_has_invalid_field(self):
        session_doc = common.get_fake_session_0()
        session_doc['session_id'] = 44
        self.assertRaises(exceptions.BadDataFormat, utils.SessionDoc.validate,
                          session_doc)

    def test_validate_patch_raises_when_doc_has_invalid_field(self):
        session_doc = common.get_fake_session_0()
        session_doc['session_id'] = 44
        self.assertRaises(exceptions.BadDataFormat,
                          utils.SessionDoc.validate_patch, session_doc)

    def test_createpatch_pops_sessionid_and_userid(self):
        session_doc = common.get_fake_session_0()
        res_doc = utils.SessionDoc.create_patch(session_doc)
        self.assertFalse('session_id' in res_doc)
        self.assertFalse('user_id' in res_doc)

    def test_createpatch_raises_BadDataFormat_when_patch_has_invalid_field(
            self):
        session_doc = common.get_fake_session_0()
        session_doc['session_tag'] = 'ouch'
        self.assertRaises(exceptions.BadDataFormat,
                          utils.SessionDoc.create_patch, session_doc)

    @patch('freezer_api.common.utils.uuid')
    def test_create_inserts_correct_uuid(self, mock_uuid):
        mock_uuid.uuid4.return_value = mock_uuid
        mock_uuid.hex = 'hotforteacher'
        session_doc = common.get_fake_session_0()
        res_doc = utils.SessionDoc.create(session_doc, 'dude')
        self.assertEqual('dude', res_doc['user_id'])
        self.assertEqual('hotforteacher', res_doc['session_id'])

    @patch('freezer_api.common.utils.uuid')
    def test_create_raises_BadDataFormat_when_isvalid_fails(self, mock_uuid):
        mock_uuid.uuid4.return_value = mock_uuid
        mock_uuid.hex = 'hotforteacher'
        session_doc = common.get_fake_session_0()
        session_doc['time_started'] = 'ouch'
        self.assertRaises(exceptions.BadDataFormat, utils.SessionDoc.create,
                          session_doc, 'dude')



# (c) Copyright 2018 ZTE Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Tests for manipulating Backup via the DB API"""


import copy
from unittest import mock
from unittest.mock import patch

from freezer_api.common import exceptions as freezer_api_exc
from freezer_api.tests.unit import common
from freezer_api.tests.unit.sqlalchemy import base


class DbBackupTestCase(base.DbTestCase):

    def setUp(self):
        super(DbBackupTestCase, self).setUp()
        self.fake_backup_metadata = common.get_fake_backup_metadata()
        self.fake_user_id = common.fake_data_0_user_id
        self.fake_user_name = common.fake_data_0_user_name
        self.fake_project_id = common.fake_data_0_project_id

    def test_add_and_get_backup(self):
        backup_doc = copy.deepcopy(self.fake_backup_metadata)
        backup_id = self.dbapi.add_backup(user_id=self.fake_user_id,
                                          user_name=self.fake_user_name,
                                          doc=backup_doc,
                                          project_id=self.fake_project_id)
        self.assertIsNotNone(backup_id)

        result = self.dbapi.get_backup(project_id=self.fake_project_id,
                                       user_id=self.fake_user_id,
                                       backup_id=backup_id)
        self.assertIsNotNone(result)

        self.assertEqual(result.get('user_name'),
                         self.fake_user_name)

        self.assertEqual(result.get('client_id'),
                         self.fake_backup_metadata.get('client_id'))

        self.assertEqual(result.get('user_id'),
                         self.fake_user_id)

        backup_metadata = result.get('backup_metadata')

        self.assertEqual(backup_metadata,
                         self.fake_backup_metadata)

    def test_add_and_delete_backup(self):
        backup_doc = copy.deepcopy(self.fake_backup_metadata)
        backup_id = self.dbapi.add_backup(user_id=self.fake_user_id,
                                          user_name=self.fake_user_name,
                                          doc=backup_doc,
                                          project_id=self.fake_project_id)

        self.assertIsNotNone(backup_id)

        result = self.dbapi.delete_backup(project_id=self.fake_project_id,
                                          user_id=self.fake_user_id,
                                          backup_id=backup_id)

        self.assertIsNotNone(result)

        self.assertEqual(result, backup_id)

        result = self.dbapi.get_backup(project_id=self.fake_project_id,
                                       user_id=self.fake_user_id,
                                       backup_id=backup_id)
        self.assertEqual(len(result), 0)

    def test_add_and_search_backup(self):
        count = 0
        backupids = []
        while (count < 20):
            backup_doc = copy.deepcopy(self.fake_backup_metadata)
            backup_id = self.dbapi.add_backup(user_id=self.fake_user_id,
                                              user_name=self.fake_user_name,
                                              doc=backup_doc,
                                              project_id=self.fake_project_id)

            self.assertIsNotNone(backup_id)
            backupids.append(backup_id)
            count += 1

        result = self.dbapi.search_backup(project_id=self.fake_project_id,
                                          user_id=self.fake_user_id,
                                          limit=10,
                                          offset=0)

        self.assertIsNotNone(result)

        self.assertEqual(len(result), 10)

        for index in range(len(result)):
            backupmap = result[index]
            self.assertEqual(backupids[index], backupmap['backup_id'])

    @patch('freezer_api.db.sqlalchemy.api.get_backup')
    def test_raise_add_backup_exist(self, mock_get_backup):
        mock_get_backup.return_value = mock.MagicMock()
        self.assertRaises(freezer_api_exc.DocumentExists,
                          self.dbapi.add_backup, self.fake_user_id,
                          '12343', self.fake_backup_metadata,
                          project_id=self.fake_project_id)

    @patch('freezer_api.common.elasticv2_utils.BackupMetadataDoc')
    @patch('freezer_api.common.utils.BackupMetadataDoc')
    def test_raise_add_backup(self, mock1_BackupMetadataDoc,
                              mock_BackupMetadataDoc):
        mock1_BackupMetadataDoc().is_valid.return_value = None
        mock_BackupMetadataDoc().is_valid.return_value = None
        mock_doc = mock.MagicMock()
        self.assertRaises(freezer_api_exc.BadDataFormat,
                          self.dbapi.add_backup, self.fake_user_id, '12343',
                          mock_doc,
                          project_id=self.fake_project_id)



"""Freezer swift.py related tests

(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

import falcon
from unittest import mock

from oslo_serialization import jsonutils as json

from freezer_api.api import v2
from freezer_api.api import versions
from freezer_api.tests.unit import common


class TestVersionResource(common.FreezerBaseTestCase):

    def setUp(self):
        super(TestVersionResource, self).setUp()
        self.resource = versions.Resource()
        self.req = mock.Mock()
        self.req.url = "{0}"

    def test_on_get_return_versions(self):
        self.resource.on_get(self.req, self.req)
        self.assertEqual(falcon.HTTP_300, self.req.status)
        expected_result = json.dumps({'versions': [v2.VERSION]})
        self.assertEqual(expected_result, self.req.data)



# -*- encoding: utf-8 -*-
# (c) Copyright 2018 ZTE Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for manipulating job via the DB API"""

import copy
from unittest import mock
from unittest.mock import patch

from freezer_api.common import exceptions as freezer_api_exc
from freezer_api.tests.unit import common
from freezer_api.tests.unit.sqlalchemy import base


class DbSessionTestCase(base.DbTestCase):

    def setUp(self):
        super(DbSessionTestCase, self).setUp()
        self.fake_session_0 = common.get_fake_session_0()
        self.fake_project_id = self.fake_session_0.get('project_id')
        self.fake_user_id = self.fake_session_0.get('user_id')
        self.fake_session_0.pop('session_id')
        self.fake_session_2 = common.get_fake_session_2()
        self.fake_session_2.pop('session_id')
        self.fake_session_3 = common.get_fake_session_3()
        self.fake_session_3.pop('session_id')
        self.fake_session_id = common.get_fake_session_id()

    def test_add_and_get_session(self):
        session_doc = copy.deepcopy(self.fake_session_0)
        session_id = self.dbapi.add_session(project_id=self.fake_session_0.
                                            get('project_id'),
                                            user_id=self.fake_session_0.
                                            get('user_id'),
                                            doc=session_doc)
        self.assertIsNotNone(session_id)
        result = self.dbapi.get_session(project_id=self.fake_session_0.
                                        get('project_id'),
                                        user_id=self.fake_session_0.
                                        get('user_id'),
                                        session_id=session_id)
        self.assertIsNotNone(result)
        self.assertEqual(result.get('session_tag'),
                         self.fake_session_0.get('session_tag'))
        self.assertEqual(result.get('description'),
                         self.fake_session_0.get('description'))
        self.assertEqual(result.get('hold_off'),
                         self.fake_session_0.get('hold_off'))
        self.assertEqual(result.get('time_start'),
                         self.fake_session_0.get('time_start'))
        self.assertEqual(result.get('time_end'),
                         self.fake_session_0.get('time_end'))
        self.assertEqual(result.get('user_id'),
                         self.fake_session_0.get('user_id'))
        self.assertEqual(result.get('project_id'),
                         self.fake_session_0.get('project_id'))
        self.assertEqual(result.get('schedule'),
                         self.fake_session_0.get('schedule'))

    def test_add_and_delete_session(self):
        session_doc = copy.deepcopy(self.fake_session_0)
        session_id = self.dbapi.add_session(project_id=self.fake_session_0.
                                            get('project_id'),
                                            user_id=self.fake_session_0.
                                            get('user_id'),
                                            doc=session_doc)
        self.assertIsNotNone(session_id)

        result = self.dbapi.delete_session(project_id=self.fake_session_0.
                                           get('project_id'),
                                           user_id=self.fake_session_0.
                                           get('user_id'),
                                           session_id=session_id)

        self.assertIsNotNone(result)
        self.assertEqual(result, session_id)
        result = self.dbapi.get_session(project_id=self.fake_session_0.
                                        get('project_id'),
                                        user_id=self.fake_session_0.
                                        get('user_id'),
                                        session_id=session_id)
        self.assertEqual(len(result), 0)

    def test_add_and_update_session(self):
        session_doc = copy.deepcopy(self.fake_session_0)
        session_id = self.dbapi.add_session(project_id=self.fake_session_0.
                                            get('project_id'),
                                            user_id=self.fake_session_0.
                                            get('user_id'),
                                            doc=session_doc)
        self.assertIsNotNone(session_id)

        patch_doc = copy.deepcopy(self.fake_session_2)
        result = self.dbapi.update_session(user_id=self.fake_session_0.
                                           get('user_id'),
                                           session_id=session_id,
                                           patch_doc=patch_doc,
                                           project_id=self.fake_session_0.
                                           get('project_id'))
        self.assertIsNotNone(result)
        self.assertEqual(result, 0)

        result = self.dbapi.get_session(project_id=self.fake_session_0.
                                        get('project_id'),
                                        user_id=self.fake_session_0.
                                        get('user_id'),
                                        session_id=session_id)
        self.assertIsNotNone(result)
        self.assertEqual(result.get('description'),
                         self.fake_session_2.get('description'))
        self.assertEqual(result.get('hold_off'),
                         self.fake_session_2.get('hold_off'))
        self.assertEqual(result.get('schedule').get('schedule_date'),
                         self.fake_session_2.get('schedule').
                         get('schedule_date'))

    def test_add_and_replace_session(self):
        session_doc = copy.deepcopy(self.fake_session_0)
        session_id = self.dbapi.add_session(project_id=self.fake_session_0.
                                            get('project_id'),
                                            user_id=self.fake_session_0.
                                            get('user_id'),
                                            doc=session_doc)
        self.assertIsNotNone(session_id)

        patch_doc = copy.deepcopy(self.fake_session_2)
        result = self.dbapi.replace_session(user_id=self.
                                            fake_session_2.get('user_id'),
                                            session_id=session_id,
                                            doc=patch_doc,
                                            project_id=self.fake_session_2.
                                            get('project_id'))

        self.assertIsNotNone(result)
        self.assertEqual(result, session_id)

        result = self.dbapi.get_session(project_id=self.fake_session_2.
                                        get('project_id'),
                                        user_id=self.fake_session_2.
                                        get('user_id'),
                                        session_id=session_id)

        self.assertIsNotNone(result)
        self.assertEqual(result.get('description'),
                         self.fake_session_2.get('description'))
        self.assertEqual(result.get('hold_off'),
                         self.fake_session_2.get('hold_off'))
        self.assertEqual(result.get('schedule').get('schedule_date'),
                         self.fake_session_2.get('schedule').
                         get('schedule_date'))

        patch_doc1 = copy.deepcopy(self.fake_session_0)
        result = self.dbapi.replace_session(user_id=self.
                                            fake_session_0.get('user_id'),
                                            session_id=self.fake_session_id,
                                            doc=patch_doc1,
                                            project_id=self.fake_session_2.
                                            get('project_id'))
        self.assertIsNotNone(result)
        result = self.dbapi.get_session(project_id=self.fake_session_2.
                                        get('project_id'),
                                        user_id=self.fake_session_2.
                                        get('user_id'),
                                        session_id=self.fake_session_id)
        self.assertEqual(result.get('session_id'), self.fake_session_id)

    def test_session_list_without_search(self):
        count = 0
        sessionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_session_3)
            session_id = self.dbapi.add_session(project_id=self.fake_session_3.
                                                get('project_id'),
                                                user_id=self.fake_session_3.
                                                get('user_id'),
                                                doc=doc)
            self.assertIsNotNone(session_id)
            sessionids.append(session_id)
            count += 1

        result = self.dbapi.search_session(user_id=self.fake_session_3.
                                           get('user_id'),
                                           project_id=self.fake_session_3.
                                           get('project_id'),
                                           offset=0,
                                           limit=10)
        self.assertIsNotNone(result)
        self.assertEqual(len(result), 10)

        for index in range(len(result)):
            sessionmap = result[index]
            self.assertEqual(sessionids[index], sessionmap['session_id'])

    def test_session_list_with_search_match_and_match_not(self):
        count = 0
        sessionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_session_3)
            if count in [0, 4, 8, 12, 16]:
                doc['hold_off'] = 100
                if count in [4, 12]:
                    doc['schedule']['schedule_date'] = \
                        '2018-12-12T00:00:00'
            session_id = self.dbapi.add_session(project_id=self.fake_session_3.
                                                get('project_id'),
                                                user_id=self.fake_session_3.
                                                get('user_id'),
                                                doc=doc)
            self.assertIsNotNone(session_id)
            sessionids.append(session_id)
            count += 1
        search_opt = {'match': [{'hold_off': 100}],
                      'match_not':
                          [{'schedule_date': '2018-12-12T00:00:00'}]
                      }
        result = self.dbapi.search_session(user_id=self.fake_session_3.
                                           get('user_id'),
                                           project_id=self.fake_session_3.
                                           get('project_id'),
                                           offset=0,
                                           limit=20,
                                           search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 3)
        for index in range(len(result)):
            sessionmap = result[index]
            self.assertEqual(100, sessionmap['hold_off'])
            self.assertEqual('2018-11-14T16:20:00',
                             sessionmap['schedule']['schedule_date'])

    def test_session_list_with_search_match_list(self):
        count = 0
        sessionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_session_3)
            if count in [0, 4, 8, 12, 16]:
                doc['hold_off'] = 100
                if count in [4, 12]:
                    doc['schedule']['schedule_date'] = '2018-12-12T00:00:00'
            session_id = self.dbapi.add_session(project_id=self.fake_session_3.
                                                get('project_id'),
                                                user_id=self.fake_session_3.
                                                get('user_id'),
                                                doc=doc)
            self.assertIsNotNone(session_id)
            sessionids.append(session_id)
            count += 1
        search_opt = {'match': [{'hold_off': 100},
                                {'schedule_date': '2018-12-12T00:00:00'}]}
        result = self.dbapi.search_session(user_id=self.fake_session_3.
                                           get('user_id'),
                                           project_id=self.fake_session_3.
                                           get('project_id'),
                                           offset=0,
                                           limit=20,
                                           search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 2)
        for index in range(len(result)):
            sessionmap = result[index]
            self.assertEqual(100, sessionmap['hold_off'])
            self.assertEqual('2018-12-12T00:00:00',
                             sessionmap['schedule']['schedule_date'])

    def test_session_list_with_search_match_not_list(self):
        count = 0
        sessionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_session_3)
            if count in [0, 4, 8, 12, 16]:
                doc['hold_off'] = 100
                if count in [4, 12]:
                    doc['schedule']['schedule_date'] = '2018-12-12T00:00:00'
            session_id = self.dbapi.add_session(project_id=self.fake_session_3.
                                                get('project_id'),
                                                user_id=self.fake_session_3.
                                                get('user_id'),
                                                doc=doc)
            self.assertIsNotNone(session_id)
            sessionids.append(session_id)
            count += 1
        search_opt = {'match_not': [{'schedule_date': '2018-11-14T16:20:00'},
                                    {'hold_off': 60}]}
        result = self.dbapi.search_session(user_id=self.fake_session_3.
                                           get('user_id'),
                                           project_id=self.fake_session_3.
                                           get('project_id'),
                                           offset=0,
                                           limit=20,
                                           search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 2)
        for index in range(len(result)):
            sessionmap = result[index]
            self.assertEqual(100, sessionmap['hold_off'])
            self.assertEqual('2018-12-12T00:00:00',
                             sessionmap['schedule']['schedule_date'])

    def test_session_list_with_search_with_all_opt_one_match(self):
        count = 0
        sessionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_session_3)
            if count in [0, 4, 8, 12, 16]:
                doc['hold_off'] = 100
            session_id = self.dbapi.add_session(project_id=self.fake_session_3.
                                                get('project_id'),
                                                user_id=self.fake_session_3.
                                                get('user_id'),
                                                doc=doc)
            self.assertIsNotNone(session_id)
            sessionids.append(session_id)
            count += 1

        search_opt = {'match': [{'_all': '[{"hold_off": 100}]'}]}
        result = self.dbapi.search_session(user_id=self.fake_session_3.
                                           get('user_id'),
                                           project_id=self.fake_session_3.
                                           get('project_id'),
                                           offset=0,
                                           limit=20,
                                           search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 5)
        for index in range(len(result)):
            sessionmap = result[index]
            self.assertEqual(100, sessionmap['hold_off'])

    def test_session_list_with_search_with_all_opt_two_matches(self):
        count = 0
        sessionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_session_3)
            if count in [0, 4, 8, 12, 16]:
                doc['hold_off'] = 100
            if count in [4, 12]:
                doc['schedule']['schedule_date'] = '2018-12-12T00:00:00'
            session_id = self.dbapi.add_session(project_id=self.fake_session_3.
                                                get('project_id'),
                                                user_id=self.fake_session_3.
                                                get('user_id'),
                                                doc=doc)
            self.assertIsNotNone(session_id)
            sessionids.append(session_id)
            count += 1

        search_opt = {'match': [{'_all': '[{"hold_off": 100},'
                                         '{"schedule_date": '
                                         '"2018-12-12T00:00:00"}]'}]}
        result = self.dbapi.search_session(user_id=self.fake_session_3.
                                           get('user_id'),
                                           project_id=self.fake_session_3.
                                           get('project_id'),
                                           offset=0,
                                           limit=20,
                                           search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 2)
        for index in range(len(result)):
            sessionmap = result[index]
            self.assertEqual(100, sessionmap['hold_off'])
            self.assertEqual('2018-12-12T00:00:00',
                             sessionmap['schedule']['schedule_date'])

    def test_session_list_with_search_error_all_opt_return_alltuples(self):
        count = 0
        sessionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_session_3)
            if count in [0, 4, 8, 12, 16]:
                doc['hold_off'] = 100
            session_id = self.dbapi.add_session(project_id=self.fake_session_3.
                                                get('project_id'),
                                                user_id=self.fake_session_3.
                                                get('user_id'),
                                                doc=doc)
            self.assertIsNotNone(session_id)
            sessionids.append(session_id)
            count += 1

        search_opt = {'match': [{'_all': '{"hold_off": 100}'}]}
        result = self.dbapi.search_session(user_id=self.fake_session_3.
                                           get('user_id'),
                                           project_id=self.fake_session_3.
                                           get('project_id'),
                                           offset=0,
                                           limit=20,
                                           search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 20)

        search_opt = {'match': [{'_all': 'hold_off=100'}]}
        result = self.dbapi.search_session(user_id=self.fake_session_3.
                                           get('user_id'),
                                           project_id=self.fake_session_3.
                                           get('project_id'),
                                           offset=0,
                                           limit=20,
                                           search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 20)

    def test_session_list_with_search_and_offset_and_limit(self):
        count = 0
        sessionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_session_3)
            if count in [0, 4, 8, 12, 16]:
                doc['hold_off'] = 100
            session_id = self.dbapi.add_session(project_id=self.fake_session_3.
                                                get('project_id'),
                                                user_id=self.fake_session_3.
                                                get('user_id'),
                                                doc=doc)
            self.assertIsNotNone(session_id)
            sessionids.append(session_id)
            count += 1
        # There are 5 records.
        search_opt = {'match': [{'_all': '[{"hold_off": 100}]'}]}
        # First, we can get 3 tuples
        result = self.dbapi.search_session(user_id=self.fake_session_3.
                                           get('user_id'),
                                           project_id=self.fake_session_3.
                                           get('project_id'),
                                           offset=0,
                                           limit=3,
                                           search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 3)
        for index in range(len(result)):
            sessionmap = result[index]
            self.assertEqual(100, sessionmap['hold_off'])
        # Second, we can get 2 tuples
        result = self.dbapi.search_session(user_id=self.fake_session_3.
                                           get('user_id'),
                                           project_id=self.fake_session_3.
                                           get('project_id'),
                                           offset=3,
                                           limit=3,
                                           search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 2)
        for index in range(len(result)):
            sessionmap = result[index]
            self.assertEqual(100, sessionmap['hold_off'])

    @patch('freezer_api.db.sqlalchemy.api.get_session')
    def test_raise_add_session_exist(self, mock_get_session):
        mock_get_session.return_value = mock.MagicMock()
        self.assertRaises(freezer_api_exc.DocumentExists,
                          self.dbapi.add_session, self.fake_user_id,
                          self.fake_session_0,
                          project_id=self.fake_project_id)

    def test_raise_update_session_noexist(self):
        self.assertRaises(freezer_api_exc.DocumentNotFound,
                          self.dbapi.update_session, self.fake_user_id,
                          self.fake_session_id,
                          self.fake_session_0,
                          project_id=self.fake_project_id)



# (c) Copyright 2018 ZTE Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import falcon
from unittest import mock

from freezer_api.api.v2 import clients as v2_clients
from freezer_api.common import exceptions
from freezer_api.tests.unit import common


class TestClientsCollectionResource(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestClientsCollectionResource, self).setUp()
        self.mock_db = mock.Mock()
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.get_header.return_value = common.fake_data_0_user_id
        self.mock_req.status = falcon.HTTP_200
        self.resource = v2_clients.ClientsCollectionResource(self.mock_db)
        self.mock_json_body = mock.Mock()
        self.mock_json_body.return_value = {}
        self.resource.json_body = self.mock_json_body

    def test_on_get_return_empty_list(self):
        self.mock_db.get_client.return_value = []
        expected_result = {'clients': []}
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_client_info_0['project_id'])
        result = self.mock_req.body
        self.assertEqual(expected_result, result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_get_return_correct_list(self):
        self.mock_db.get_client.return_value = [common.fake_client_entry_0,
                                                common.fake_client_entry_1]
        expected_result = {'clients': [common.fake_client_entry_0,
                                       common.fake_client_entry_1]}
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_client_info_0['project_id'])
        result = self.mock_req.body
        self.assertEqual(expected_result, result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_post_raises_when_missing_body(self):
        self.mock_db.add_client.return_value = common.fake_client_info_0[
            'client_id']
        self.assertRaises(exceptions.BadDataFormat, self.resource.on_post,
                          self.mock_req, self.mock_req,
                          common.fake_client_info_0['project_id'])

    def test_on_post_inserts_correct_data(self):
        self.mock_json_body.return_value = common.fake_client_info_0
        self.mock_db.add_client.return_value = common.fake_client_info_0[
            'client_id']
        expected_result = {'client_id': common.fake_client_info_0['client_id']}
        self.resource.on_post(self.mock_req, self.mock_req,
                              common.fake_client_info_0['project_id'])
        self.assertEqual(falcon.HTTP_201, self.mock_req.status)
        self.assertEqual(expected_result, self.mock_req.body)


class TestClientsResource(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestClientsResource, self).setUp()
        self.mock_db = mock.Mock()
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.get_header.return_value = common.fake_data_0_user_id
        self.mock_req.status = falcon.HTTP_200
        self.resource = v2_clients.ClientsResource(self.mock_db)
        self.mock_json_body = mock.Mock()
        self.mock_json_body.return_value = {}
        self.resource.json_body = self.mock_json_body

    def test_create_resource(self):
        self.assertIsInstance(self.resource, v2_clients.ClientsResource)

    def test_on_get_return_no_result_and_404_when_not_found(self):
        self.mock_db.get_client.return_value = []
        self.mock_req.body = None
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_client_info_0['project_id'],
                             common.fake_client_info_0['client_id'])
        self.assertIsNone(self.mock_req.body)
        self.assertEqual(falcon.HTTP_404, self.mock_req.status)

    def test_on_get_return_correct_data(self):
        self.mock_db.get_client.return_value = [common.fake_client_entry_0]
        expected_result = common.fake_client_entry_0
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_client_info_0['project_id'],
                             common.fake_client_info_0['client_id'])
        result = self.mock_req.body
        self.assertEqual(expected_result, result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_delete_removes_proper_data(self):
        self.resource.on_delete(self.mock_req, self.mock_req,
                                common.fake_client_info_0['project_id'],
                                common.fake_client_info_0['client_id'])
        result = self.mock_req.body
        expected_result = {'client_id': common.fake_client_info_0['client_id']}
        self.assertEqual(falcon.HTTP_204, self.mock_req.status)
        self.assertEqual(expected_result, result)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import threading

from stevedore import driver

from oslo_config import cfg
from oslo_db import api as db_api
from oslo_db.sqlalchemy import migration
from oslo_log import log

from freezer_api.db import base as db_base
from freezer_api.db.sqlalchemy import api as db_session
from freezer_api.db.sqlalchemy import models


INIT_VERSION = 0

_IMPL = None
_LOCK = threading.Lock()

CONF = cfg.CONF
LOG = log.getLogger(__name__)


MIGRATE_REPO_PATH = os.path.join(
    os.path.abspath(os.path.dirname(__file__)),
    'migrate_repo',
)

_BACKEND_MAPPING = {'sqlalchemy': 'freezer_api.db.sqlalchemy.api'}


class SQLDriver(db_base.DBDriver):

    def __init__(self, backend):
        super(SQLDriver, self).__init__(backend)
        self.IMPL = db_api.DBAPI.from_config(CONF, _BACKEND_MAPPING)
        self._engine = None

    def get_engine(self):
        if not self._engine:
            self._engine = db_session.get_engine()
        return self._engine

    def get_api(self):
        self.get_engine()
        return self.IMPL

    def get_backend(self):
        global _IMPL
        if _IMPL is None:
            with _LOCK:
                if _IMPL is None:
                    _IMPL = driver.DriverManager(
                        "freezer.database.migration_backend",
                        cfg.CONF.database.backend).driver
        return _IMPL

    def db_sync(self, version=None, init_version=INIT_VERSION, engine=None):
        """Migrate the database to `version` or the most recent version."""

        if not self._engine:
            self._engine = self.get_engine()
        return migration.db_sync(engine=self._engine,
                                 abs_path=MIGRATE_REPO_PATH,
                                 version=version,
                                 init_version=init_version)

    def db_show(self):
        if not self._engine:
            self._engine = self.get_engine()
        return models.get_tables(self._engine)

    def db_remove(self):
        if not self._engine:
            self._engine = self.get_engine()
        models.unregister_models(self._engine)

    def name(self):
        return "sqlalchemy"



"""Freezer swift.py related tests

Copyright 2018 ZTE Corporation.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

import random

import falcon
from unittest import mock
from unittest.mock import patch

from oslo_serialization import jsonutils as json

from freezer_api.api.v2 import jobs as v2_jobs
from freezer_api.common import exceptions
from freezer_api.tests.unit import common


class TestJobsBaseResource(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestJobsBaseResource, self).setUp()
        self.mock_db = mock.Mock()
        self.resource = v2_jobs.JobsBaseResource(self.mock_db)

    def test_get_action_returns_found_action(self):
        self.mock_db.get_action.return_value = 'awesome_result'
        result = self.resource.get_action('project-id', 'user-id', 'action-id')
        self.assertEqual('awesome_result', result)

    def test_get_action_returns_none_when_action_not_found(self):
        self.mock_db.get_action.side_effect = exceptions.DocumentNotFound(
            'regular test failure')
        result = self.resource.get_action('project-id', 'user-id', 'action-id')
        self.assertIsNone(result)

    def test_update_actions_in_job_no_action_id(self):
        self.resource.get_action = mock.Mock()
        self.resource.get_action.return_value = None
        action_doc = {
            # "action_id": "ottonero",
            "freezer_action": {
                "mode": "mysql",
                "container": "freezer_backup_test"
            },
            "max_retries": 3
        }
        job_doc = {"job_actions": [action_doc.copy()],
                   "description": "three actions backup"
                   }
        self.resource.update_actions_in_job('tecs', 'duder', job_doc=job_doc)
        self.mock_db.add_action.assert_called_with(project_id='tecs',
                                                   user_id='duder',
                                                   doc=action_doc)

    def test_update_actions_in_job_action_id_not_found(self):
        self.resource.get_action = mock.Mock()
        self.resource.get_action.return_value = None
        action_doc = {
            "action_id": "ottonero",
            "freezer_action": {
                "mode": "mysql",
                "container": "freezer_backup_test"
            },
            "max_retries": 3
        }
        job_doc = {"job_actions": [action_doc.copy()],
                   "description": "three actions backup"
                   }
        self.resource.update_actions_in_job('tecs', 'duder', job_doc=job_doc)
        self.mock_db.add_action.assert_called_with(project_id='tecs',
                                                   user_id='duder',
                                                   doc=action_doc)

    def test_update_actions_in_job_action_id_found_and_same_action(self):
        self.resource.get_action = mock.Mock()
        action_doc = {
            "action_id": "ottonero",
            "freezer_action": {
                "mode": "mysql",
                "container": "freezer_backup_test"
            },
            "max_retries": 3
        }
        job_doc = {"job_actions": [action_doc.copy()],
                   "description": "three actions backup"
                   }
        self.resource.get_action.return_value = action_doc.copy()
        self.resource.update_actions_in_job('tecs', 'duder', job_doc=job_doc)
        self.mock_db.add_action.assert_not_called()

    def test_update_actions_in_job_action_id_found_and_different_action(self):
        self.resource.get_action = mock.Mock()
        action_doc = {
            "action_id": "ottonero",
            "freezer_action": {
                "mode": "mysql",
                "container": "freezer_backup_test"
            },
            "max_retries": 3
        }
        job_doc = {"job_actions": [action_doc.copy()],
                   "description": "three actions backup"
                   }

        found_action = {
            "action_id": "ottonero",
            "freezer_action": {
                "mode": "mysql",
                "container": "different_drum"
            },
            "max_retries": 4
        }

        new_doc = action_doc.copy()
        new_doc['action_id'] = ''
        self.resource.get_action.return_value = found_action
        self.resource.update_actions_in_job('tecs', 'duder', job_doc=job_doc)
        self.mock_db.add_action.assert_called_with(project_id='tecs',
                                                   user_id='duder',
                                                   doc=new_doc)


class TestJobsCollectionResource(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestJobsCollectionResource, self).setUp()
        self.mock_json_body = mock.Mock()
        self.mock_json_body.return_value = {}
        self.mock_db = mock.Mock()
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.get_header.return_value = common.fake_job_0_user_id
        self.mock_req.status = falcon.HTTP_200
        self.resource = v2_jobs.JobsCollectionResource(self.mock_db)
        self.resource.json_body = self.mock_json_body

    def test_on_get_return_empty_list(self):
        self.mock_db.search_job.return_value = []
        expected_result = {'jobs': []}
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_job_0_project_id)
        result = self.mock_req.body
        self.assertEqual(expected_result, result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_get_return_correct_list(self):
        self.mock_db.search_job.return_value = [common.get_fake_job_0(),
                                                common.get_fake_job_1()]
        expected_result = {
            'jobs': [common.get_fake_job_0(), common.get_fake_job_1()]}
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_job_0_project_id)
        result = self.mock_req.body
        self.assertEqual(expected_result, result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_post_inserts_correct_data(self):
        job = common.get_fake_job_0()
        self.mock_json_body.return_value = job
        self.mock_db.add_job.return_value = 'pjiofrdslaikfunr'
        expected_result = {'job_id': 'pjiofrdslaikfunr'}
        self.resource.on_post(self.mock_req, self.mock_req,
                              common.fake_job_0_project_id)
        self.assertEqual(falcon.HTTP_201, self.mock_req.status)
        self.assertEqual(expected_result, self.mock_req.body)


class TestJobsResource(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestJobsResource, self).setUp()
        self.mock_db = mock.Mock()
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.stream.read.return_value = {}
        self.mock_req.get_header.return_value = common.fake_job_0_user_id
        self.mock_req.status = falcon.HTTP_200
        self.resource = v2_jobs.JobsResource(self.mock_db)

    def test_create_resource(self):
        self.assertIsInstance(self.resource, v2_jobs.JobsResource)

    def test_on_get_return_no_result_and_404_when_not_found(self):
        self.mock_req.body = None
        self.mock_db.get_job.return_value = None
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_job_0_project_id,
                             common.fake_job_0_job_id)
        self.assertIsNone(self.mock_req.body)
        self.assertEqual(falcon.HTTP_404, self.mock_req.status)

    def test_on_get_return_correct_data(self):
        self.mock_db.get_job.return_value = common.get_fake_job_0()
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_job_0_project_id,
                             common.fake_job_0_job_id)
        result = self.mock_req.body
        self.assertEqual(common.get_fake_job_0(), result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_delete_removes_proper_data(self):
        self.resource.on_delete(self.mock_req, self.mock_req,
                                common.fake_job_0_project_id,
                                common.fake_job_0_job_id)
        result = self.mock_req.body
        expected_result = {'job_id': common.fake_job_0_job_id}
        self.assertEqual(falcon.HTTP_204, self.mock_req.status)
        self.assertEqual(expected_result, result)

    def test_on_patch_ok_with_some_fields(self):
        new_version = random.randint(0, 99)
        self.mock_db.update_job.return_value = new_version
        patch_doc = {'some_field': 'some_value',
                     'because': 'size_matters',
                     'job_schedule': {}}
        self.mock_req.stream.read.return_value = json.dumps(patch_doc)
        expected_result = {'job_id': common.fake_job_0_job_id,
                           'version': new_version}
        self.resource.update_actions_in_job = mock.Mock()
        self.resource.on_patch(self.mock_req, self.mock_req,
                               common.fake_job_0_project_id,
                               common.fake_job_0_job_id)
        self.mock_db.update_job.assert_called_with(
            project_id=common.fake_job_0_project_id,
            user_id=common.fake_job_0_user_id,
            job_id=common.fake_job_0_job_id,
            patch_doc=patch_doc)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)
        result = self.mock_req.body
        self.assertEqual(expected_result, result)

    def test_on_post_ok(self):
        new_version = random.randint(0, 99)
        self.mock_db.replace_job.return_value = new_version
        job = common.get_fake_job_0()
        self.mock_req.stream.read.return_value = json.dumps(job)
        expected_result = {'job_id': common.fake_job_0_job_id,
                           'version': new_version}

        self.resource.on_post(self.mock_req, self.mock_req,
                              common.fake_job_0_project_id,
                              common.fake_job_0_job_id)
        self.assertEqual(falcon.HTTP_201, self.mock_req.status)
        self.assertEqual(expected_result, self.mock_req.body)

    def test_on_post_raises_when_db_replace_job_raises(self):
        self.mock_db.replace_job.side_effect = exceptions.AccessForbidden(
            'regular test failure')
        job = common.get_fake_job_0()
        self.mock_req.stream.read.return_value = json.dumps(job)
        self.assertRaises(exceptions.AccessForbidden, self.resource.on_post,
                          self.mock_req,
                          self.mock_req,
                          common.fake_job_0_project_id,
                          common.fake_job_0_job_id)


class TestJobsEvent(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestJobsEvent, self).setUp()
        self.mock_db = mock.Mock()
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.get_header.return_value = common.fake_session_0[
            'user_id']
        self.mock_req.status = falcon.HTTP_200
        self.resource = v2_jobs.JobsEvent(self.mock_db)
        self.mock_json_body = mock.Mock()
        self.mock_json_body.return_value = {}
        self.resource.json_body = self.mock_json_body

    def test_create_resource(self):
        self.assertIsInstance(self.resource, v2_jobs.JobsEvent)

    def test_on_post_raises_when_unable_to_read_event_from_body(self):
        self.mock_json_body.return_value = {}
        self.assertRaises(exceptions.BadDataFormat, self.resource.on_post,
                          self.mock_req,
                          self.mock_req,
                          'my_project_id',
                          'my_job_id')

    def test_on_post_start_event_ok(self):
        new_version = random.randint(0, 99)
        self.mock_db.get_job.return_value = {
            'job_schedule': {
                'status': 'stop'
            }
        }
        self.mock_db.replace_job.return_value = new_version
        event = {"start": None}
        self.mock_json_body.return_value = event
        expected_result = {'result': 'success'}
        self.resource.on_post(self.mock_req, self.mock_req,
                              'my_project_id', 'my_job_id')
        self.assertEqual(falcon.HTTP_202, self.mock_req.status)
        self.assertEqual(expected_result, self.mock_req.body)


class TestJobs(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestJobs, self).setUp()

    def _test_job_start(self, status, event, response, need_update):
        job_schedule = {}
        if status is not None:
            job_schedule['status'] = status
        if event is not None:
            job_schedule['event'] = event
        job = v2_jobs.Job({'job_schedule': job_schedule})
        res = job.start()
        self.assertEqual(response, res)
        self.assertEqual(need_update, job.need_update)

    def _test_job_stop(self, status, event, response, need_update):
        job_schedule = {}
        if status is not None:
            job_schedule['status'] = status
        if event is not None:
            job_schedule['event'] = event
        job = v2_jobs.Job({'job_schedule': job_schedule})
        res = job.stop()
        self.assertEqual(response, res)
        self.assertEqual(need_update, job.need_update)

    def _test_job_abort(self, status, event, response, need_update):
        job_schedule = {}
        if status is not None:
            job_schedule['status'] = status
        if event is not None:
            job_schedule['event'] = event
        job = v2_jobs.Job({'job_schedule': job_schedule})
        res = job.abort()
        self.assertEqual(response, res)
        self.assertEqual(need_update, job.need_update)

    def test_start_scheduled_unstarted_job(self):
        self._test_job_start(status='scheduled',
                             event=None,
                             response='success',
                             need_update=True)

    def test_start_scheduled_started_job(self):
        self._test_job_start(status='scheduled',
                             event='start',
                             response='start already requested',
                             need_update=False)

    def test_start_running_unstarted_job(self):
        self._test_job_start(status='running',
                             event=None,
                             response='success',
                             need_update=True)

    def test_start_running_started_job(self):
        self._test_job_start(status='running',
                             event='start',
                             response='start already requested',
                             need_update=False)

    def test_start_stopped_unstarted_job(self):
        self._test_job_start(status='stop',
                             event=None,
                             response='success',
                             need_update=True)

    def test_start_stopped_started_job(self):
        self._test_job_start(status='stop',
                             event='start',
                             response='start already requested',
                             need_update=False)

    def test_start_completed_unstarted_job(self):
        self._test_job_start(status='completed',
                             event=None,
                             response='success',
                             need_update=True)

    def test_start_completed_started_job(self):
        self._test_job_start(status='completed',
                             event='start',
                             response='start already requested',
                             need_update=False)

    def test_start_emptystatus_unstarted_job(self):
        self._test_job_start(status='',
                             event=None,
                             response='success',
                             need_update=True)

    def test_start_emptystatus_started_job(self):
        self._test_job_start(status=None,
                             event='start',
                             response='start already requested',
                             need_update=False)

    def test_start_nostatus_unstarted_job(self):
        self._test_job_start(status=None,
                             event=None,
                             response='success',
                             need_update=True)

    def test_stop_scheduled_unstopped_job(self):
        self._test_job_stop(status='scheduled',
                            event=None,
                            response='success',
                            need_update=True)

    def test_stop_scheduled_stopped_job(self):
        self._test_job_stop(status='scheduled',
                            event='stop',
                            response='stop already requested',
                            need_update=False)

    def test_stop_running_unstopped_job(self):
        self._test_job_stop(status='running',
                            event=None,
                            response='success',
                            need_update=True)

    def test_stop_running_stopped_job(self):
        self._test_job_stop(status='running',
                            event='stop',
                            response='stop already requested',
                            need_update=False)

    def test_stop_stopped_unstopped_job(self):
        self._test_job_stop(status='stop',
                            event=None,
                            response='success',
                            need_update=True)

    def test_stop_stopped_stopped_job(self):
        self._test_job_stop(status='stop',
                            event='stop',
                            response='stop already requested',
                            need_update=False)

    def test_stop_completed_unstopped_job(self):
        self._test_job_stop(status='completed',
                            event=None,
                            response='success',
                            need_update=True)

    def test_stop_completed_stopped_job(self):
        self._test_job_stop(status='completed',
                            event='stop',
                            response='stop already requested',
                            need_update=False)

    def test_stop_emptystatus_unstopped_job(self):
        self._test_job_stop(status='',
                            event=None,
                            response='success',
                            need_update=True)

    def test_stop_emptystatus_stopped_job(self):
        self._test_job_stop(status=None,
                            event='stop',
                            response='stop already requested',
                            need_update=False)

    def test_stop_nostatus_unstopped_job(self):
        self._test_job_stop(status=None,
                            event=None,
                            response='success',
                            need_update=True)

    def test_abort_scheduled_unaborted_job(self):
        self._test_job_abort(status='scheduled',
                             event=None,
                             response='success',
                             need_update=True)

    def test_abort_scheduled_abortped_job(self):
        self._test_job_abort(status='scheduled',
                             event='abort',
                             response='abort already requested',
                             need_update=False)

    def test_abort_running_unaborted_job(self):
        self._test_job_abort(status='running',
                             event=None,
                             response='success',
                             need_update=True)

    def test_abort_running_abortped_job(self):
        self._test_job_abort(status='running',
                             event='abort',
                             response='abort already requested',
                             need_update=False)

    def test_abort_abortped_unaborted_job(self):
        self._test_job_abort(status='abort',
                             event=None,
                             response='success',
                             need_update=True)

    def test_abort_abortped_abortped_job(self):
        self._test_job_abort(status='abort',
                             event='abort',
                             response='abort already requested',
                             need_update=False)

    def test_abort_completed_unaborted_job(self):
        self._test_job_abort(status='completed',
                             event=None,
                             response='success',
                             need_update=True)

    def test_abort_completed_abortped_job(self):
        self._test_job_abort(status='completed',
                             event='abort',
                             response='abort already requested',
                             need_update=False)

    def test_abort_emptystatus_unaborted_job(self):
        self._test_job_abort(status='',
                             event=None,
                             response='success',
                             need_update=True)

    def test_abort_emptystatus_abortped_job(self):
        self._test_job_abort(status=None,
                             event='abort',
                             response='abort already requested',
                             need_update=False)

    def test_abort_nostatus_unaborted_job(self):
        self._test_job_abort(status=None,
                             event=None,
                             response='success',
                             need_update=True)

    @patch.object(v2_jobs.Job, 'start')
    def test_execute_start_event(self, mock_start):
        job = v2_jobs.Job({})
        job.execute_event('start', 'my_params')
        mock_start.assert_called_once_with('my_params')

    @patch.object(v2_jobs.Job, 'stop')
    def test_execute_stop_event(self, mock_stop):
        job = v2_jobs.Job({})
        job.execute_event('stop', 'my_params')
        mock_stop.assert_called_once_with('my_params')

    @patch.object(v2_jobs.Job, 'abort')
    def test_execute_abort_event(self, mock_abort):
        job = v2_jobs.Job({})
        job.execute_event('abort', 'my_params')
        mock_abort.assert_called_once_with('my_params')

    def test_execute_raises_BadDataFormat_when_event_not_implemented(self):
        job = v2_jobs.Job({})
        self.assertRaises(exceptions.BadDataFormat, job.execute_event, 'smile',
                          'my_params')

    def test_expand_action_defaults(self):
        job_doc = {
            'action_defaults': {'that_field': 'that_value'},
            'job_actions': [
                {'freezer_action': {'not_that_field': 'some_value'}},
                {'freezer_action': {'that_field': 'another_value'}}
            ]
        }
        expected_job_doc = {
            'job_actions': [
                {'freezer_action': {'not_that_field': 'some_value',
                                    'that_field': 'that_value'}},
                {'freezer_action': {'that_field': 'another_value'}}
            ],
            'job_schedule': {}
        }
        job = v2_jobs.Job(job_doc)
        self.assertEqual(expected_job_doc, job.doc)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sqlalchemy import Boolean, Column, DateTime
from sqlalchemy import Integer, MetaData, String, Table, Text
from sqlalchemy import BLOB, TIMESTAMP

CLASS_NAME = 'default'


def define_tables(meta, sqlite):
    clients = Table(
        'clients', meta,
        Column('created_at', DateTime(timezone=False)),
        Column('updated_at', DateTime(timezone=False)),
        Column('deleted_at', DateTime),
        Column('deleted', Boolean),
        Column('user_id', String(36), nullable=False),
        Column('id', String(255), primary_key=True, nullable=False),
        Column('project_id', String(36)),
        Column('client_id', String(255), nullable=False),
        Column('hostname', String(255), nullable=False),
        Column('description', String(255)),
        Column('uuid', String(36), nullable=False),
        mysql_engine='InnoDB',
        mysql_charset='utf8'
    )

    # scheduling field info is json, same as job table
    sessions = Table(
        'sessions', meta,
        Column('created_at', DateTime(timezone=False)),
        Column('updated_at', DateTime(timezone=False)),
        Column('deleted_at', DateTime),
        Column('deleted', Boolean),
        Column('id', String(36), primary_key=True, nullable=False),
        Column('session_tag', Integer, default=0),
        Column('description', String(255)),
        Column('hold_off', Integer, default=30),
        Column('schedule', Text),
        Column('job', Text),
        Column('project_id', String(36)),
        Column('user_id', String(36), nullable=False),
        Column('time_start', Integer, default=-1),
        Column('time_end', Integer, default=-1),
        Column('time_started', Integer, default=-1),
        Column('time_ended', Integer, default=-1),
        Column('status', String(255)),
        Column('result', String(255)),
        mysql_engine='InnoDB',
        mysql_charset='utf8'
    )

    # scheduling field info is json, including: {
    #  "time_ended" : 1538122522,
    #  "schedule_hour" : "16",
    #  "time_started" : 1538122500,
    #  "schedule_start_date" : "2018-09-25T15:50:03",
    # "schedule_minute" : "15",
    #  "time_created" : 1537861978,
    #  "event" : "",
    #  "status" : "scheduled",
    #  "result" : "success",
    #  "current_pid" : 14520
    # }, etc
    jobs = Table(
        'jobs', meta,
        Column('created_at', DateTime(timezone=False)),
        Column('updated_at', DateTime(timezone=False)),
        Column('deleted_at', DateTime),
        Column('deleted', Boolean),
        Column('id', String(36), primary_key=True, nullable=False),
        Column('project_id', String(36)),
        Column('user_id', String(36), nullable=False),
        Column('schedule', Text),
        Column('client_id', String(255), nullable=False),
        Column('session_id', String(36), default=''),
        Column('session_tag', Integer, default=0),
        Column('description', String(255)),
        Column('job_actions', Text),
        mysql_engine='InnoDB',
        mysql_charset='utf8'
    )

    # The field metadata is json, including :
    # nova_inst_id, engine_name, storage, remove_older_than, restore_from_date,
    # command, incremental, restore_abs_path, etc
    if sqlite:
        column_name = 'actionmode'
    else:
        column_name = 'mode'

    actions = Table(
        'actions', meta,
        Column('created_at', DateTime(timezone=False)),
        Column('updated_at', DateTime(timezone=False)),
        Column('deleted_at', DateTime),
        Column('deleted', Boolean),
        Column('id', String(36), primary_key=True, nullable=False),
        Column('action', String(255), nullable=False),
        Column('project_id', String(36)),
        Column('user_id', String(36), nullable=False),
        Column(column_name, String(255)),
        Column('src_file', String(255)),
        Column('backup_name', String(255)),
        Column('container', String(255)),
        Column('timeout', Integer),
        Column('priority', Integer),
        Column('max_retries_interval', Integer, default=6),
        Column('max_retries', Integer, default=5),
        Column('mandatory', Boolean, default=False),
        Column('log_file', String(255)),
        Column('backup_metadata', Text),
        mysql_engine='InnoDB',
        mysql_charset='utf8'
    )

    action_reports = Table(
        'action_reports', meta,
        Column('created_at', DateTime(timezone=False)),
        Column('updated_at', DateTime(timezone=False)),
        Column('deleted_at', DateTime),
        Column('deleted', Boolean),
        Column('id', String(36), primary_key=True, nullable=False),
        Column('project_id', String(36)),
        Column('user_id', String(36), nullable=False),
        Column('result', String(255)),
        Column('time_elapsed', String(255)),
        Column('report_date', TIMESTAMP),
        Column('log', BLOB),
        mysql_engine='InnoDB',
        mysql_charset='utf8'
    )

    # The field metadata is json, including :
    # nova_inst_id, engine_name, storage, remove_older_than, restore_from_date,
    # command, incremental, restore_abs_path, etc
    backups = Table(
        'backups', meta,
        Column('created_at', DateTime(timezone=False)),
        Column('updated_at', DateTime(timezone=False)),
        Column('deleted_at', DateTime),
        Column('deleted', Boolean),
        Column('id', String(36), primary_key=True, nullable=False),
        Column('job_id', String(36)),
        Column('project_id', String(36)),
        Column('user_id', String(64), nullable=False),
        Column('user_name', String(64)),
        Column('backup_metadata', Text),
        mysql_engine='InnoDB',
        mysql_charset='utf8'
    )
    return [clients, sessions, jobs, actions, action_reports, backups]


def upgrade(migrate_engine):
    meta = MetaData()
    meta.bind = migrate_engine

    # create all tables
    # Take care on create order for those with FK dependencies
    if migrate_engine.name == 'sqlite':
        sqlite = True
    else:
        sqlite = False

    tables = define_tables(meta, sqlite)
    for table in tables:
        table.create()



"""Freezer swift.py related tests

Copyright 2015 Hewlett-Packard

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

from unittest import mock

from oslo_serialization import jsonutils as json

from freezer_api.api import v1
from freezer_api.tests.unit import common


class TestHomedocResource(common.FreezerBaseTestCase):

    def setUp(self):
        self.resource = v1.homedoc.Resource()
        self.req = mock.MagicMock()
        self.req.__getitem__.side_effect = common.get_req_items
        super(TestHomedocResource, self).setUp()

    def test_on_get_return_resources_information(self):
        self.resource.on_get(self.req, self.req)
        result = json.loads(self.req.data.decode('utf-8'))
        print("TEST HOME DOC RESULT: {}".format(result))
        expected_result = v1.homedoc.HOME_DOC
        self.assertEqual(expected_result, result)



"""
(c) Copyright 2015-2016 Hewlett-Packard Enterprise Company L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os

from keystonemiddleware import opts
from oslo_config import cfg
from oslo_log import log
from oslo_policy import opts as policy_opts
from oslo_policy import policy

from freezer_api import __version__ as FREEZER_API_VERSION

CONF = cfg.CONF

AUTH_GROUP, AUTH_OPTS = opts.list_auth_token_opts()[0]

paste_deploy = [
    cfg.StrOpt('config_file', default='freezer-paste.ini',
               help='Name of the paste configuration file that defines '
                    'the available pipelines.'),
]

_DB_DRIVERS = [
    cfg.StrOpt("backend",
               help="Database backend section name. This section will "
                    "be loaded by the proper driver to connect to "
                    "the database."
               ),
    cfg.StrOpt('driver',
               # default='freezer_api.storage.elastic.ElasticSearchEngine',
               default='elasticsearch',
               help="Database driver to be used."
               )
]


def api_common_opts():

    _COMMON = [
        cfg.IPOpt('bind-host',
                  default='0.0.0.0',
                  dest='bind_host',
                  help='IP address to listen on. Default is 0.0.0.0'),
        cfg.PortOpt('bind-port',
                    default=9090,
                    dest='bind_port',
                    help='Port number to listen on. Default is 9090'
                    ),
        cfg.BoolOpt('enable_v1_api',
                    default=False,
                    help="""Default False, That is the v2 Freezer API
                    will be deployed. When this option is set
                    to ``True``, Freezer-api service will respond
                    to requests on registered endpoints conforming
                    to the v1 OpenStack Freezer api.
                    The v1 OpenStack Freezer API functions
                    as follows:
                        * Multi-tenancy is not supported under this
                         api version.
                        * Everything is user based.
                        * Freezer api v1 supports Oslo.db.
                        * Use elasticsearch db with v1 api version
                    The v2 OpenStack Freezer API functions
                    as follows:
                        * Multi-tenancy is supported under this api version.
                        * Freezer api v2 supports Oslo.db.
                        * Recommended to use oslo.db with api v2
                    Possible values:
                        * True
                        * False
                    """)
    ]

    return _COMMON


def register_db_drivers_opt():
    """Register storage configuration options"""
    # storage backend options to be registered

    opt_group = cfg.OptGroup(name='storage',
                             title='Freezer Database drivers')
    CONF.register_group(opt_group)
    CONF.register_opts(_DB_DRIVERS, group=opt_group)


def parse_args(args=[]):
    CONF.register_cli_opts(api_common_opts())
    register_db_drivers_opt()
    # register paste configuration
    paste_grp = cfg.OptGroup('paste_deploy',
                             'Paste Configuration')
    CONF.register_group(paste_grp)
    CONF.register_opts(paste_deploy, group=paste_grp)
    log.register_options(CONF)
    default_config_files = cfg.find_config_files('freezer', 'freezer-api')
    CONF(args=args,
         project='freezer-api',
         default_config_files=default_config_files,
         version=FREEZER_API_VERSION
         )
    policy.Enforcer(CONF)


def setup_logging():
    _DEFAULT_LOG_LEVELS = ['amqp=WARN', 'amqplib=WARN', 'boto=WARN',
                           'stevedore=WARN', 'oslo_log=INFO',
                           'iso8601=WARN', 'elasticsearch=WARN',
                           'requests.packages.urllib3.connectionpool=WARN',
                           'urllib3.connectionpool=WARN', 'websocket=WARN',
                           'keystonemiddleware=WARN', 'routes.middleware=WARN']
    _DEFAULT_LOGGING_CONTEXT_FORMAT = ('%(asctime)s.%(msecs)03d %(process)d '
                                       '%(levelname)s %(name)s [%(request_id)s'
                                       ' %(user_identity)s] %(instance)s '
                                       '%(message)s')
    log.set_defaults(_DEFAULT_LOGGING_CONTEXT_FORMAT, _DEFAULT_LOG_LEVELS)
    log.setup(CONF, 'freezer-api', version=FREEZER_API_VERSION)


def find_paste_config():
    """Find freezer's paste.deploy configuration file.
    freezer's paste.deploy configuration file is specified in the
    ``[paste_deploy]`` section of the main freezer-api configuration file,
    ``freezer-api.conf``.
    For example::
        [paste_deploy]
        config_file = freezer-paste.ini
    :returns: The selected configuration filename
    :raises: exception.ConfigFileNotFound
    """
    if CONF.paste_deploy.config_file:
        paste_config = CONF.paste_deploy.config_file
        if not os.path.isabs(paste_config):
            paste_config = CONF.find_file(paste_config)
    elif CONF.config_file:
        paste_config = CONF.config_file[0]
    else:
        # this provides backwards compatibility for keystone.conf files that
        # still have the entire paste configuration included, rather than just
        # a [paste_deploy] configuration section referring to an external file
        paste_config = CONF.find_file('freezer-api.conf')
    if not paste_config or not os.path.exists(paste_config):
        raise Exception('paste configuration file {0} not found !'.
                        format(paste_config))
    return paste_config


def list_opts():
    _OPTS = {
        None: api_common_opts(),
        'paste_deploy': paste_deploy,
        AUTH_GROUP: AUTH_OPTS
    }
    # update the current list of opts with db backend drivers opts
    _OPTS.update({"storage": _DB_DRIVERS})
    return _OPTS.items()


def set_lib_defaults():
    """Update default value for configuration options from other namespace.

    Example, oslo lib config options. This is needed for
    config generator tool to pick these default value changes.
    https://docs.openstack.org/oslo.config/latest/cli/
    generator.html#modifying-defaults-from-other-namespaces
    """

    # TODO(gmann): Remove setting the default value of config policy_file
    # once oslo_policy change the default value to 'policy.yaml'.
    # https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
    policy_opts.set_defaults(CONF, 'policy.yaml')



"""Freezer swift.py related tests

(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

import copy
import io
import os
import uuid

import fixtures
from oslo_config import cfg
from oslo_config import fixture as cfg_fixture
import testtools

from freezer_api.common import config
from freezer_api import policy

CONF = cfg.CONF

fake_data_0_backup_id = 'b740ed9ad2b646aba304ef54c21c6774'
fake_data_0_user_id = 'qwerty1234'
fake_data_0_user_name = 'asdffdsa'
fake_data_0_project_id = "tecs"

fake_data_0_wrapped_backup_metadata = {
    'backup_id': 'b740ed9ad2b646aba304ef54c21c6774',
    'project_id': 'tecs',
    'user_id': 'qwerty1234',
    'user_name': 'asdffdsa',
    'backup_metadata': {
        "container": "freezer_container",
        "hostname": "alpha",
        "backup_name": "important_data_backup",
        "time_stamp": 8475903425,
        "curr_backup_level": 0,
        "backup_session": 8475903425,
        "max_level": 5,
        "mode": "fs",
        "fs_real_path": "/blabla",
        "vol_snap_path": "/blablasnap",
        "total_broken_links": 0,
        "total_fs_files": 11,
        "total_directories": 2,
        "backup_size_uncompressed": 4567,
        "backup_size_compressed": 1212,
        "total_backup_session_size": 6789,
        "compression_alg": "None",
        "encrypted": "false",
        "client_os": "linux",
        "broken_links": ["link_01", "link_02"],
        "excluded_files": ["excluded_file_01", "excluded_file_02"],
        "cli": ""
    }
}


fake_data_0_backup_metadata = {
    "container": "freezer_container",
    "hostname": "alpha",
    "backup_name": "important_data_backup",
    "time_stamp": 8475903425,
    "curr_backup_level": 0,
    "backup_session": 8475903425,
    "max_level": 5,
    "mode": "fs",
    "fs_real_path": "/blabla",
    "vol_snap_path": "/blablasnap",
    "total_broken_links": 0,
    "total_fs_files": 11,
    "total_directories": 2,
    "backup_size_uncompressed": 4567,
    "backup_size_compressed": 1212,
    "total_backup_session_size": 6789,
    "compression_alg": "None",
    "encrypted": "false",
    "client_os": "linux",
    "broken_links": ["link_01", "link_02"],
    "excluded_files": ["excluded_file_01", "excluded_file_02"],
    "cli": ""
}


def get_fake_backup_metadata():
    return copy.deepcopy(fake_data_0_backup_metadata)


fake_malformed_data_0_backup_metadata = {
    "hostname": "alpha",
    "backup_name": "important_data_backup",
    "time_stamp": 8475903425,
    "curr_backup_level": 0,
    "backup_session": 8475903425,
    "max_level": 5,
    "mode": "fs",
    "fs_real_path": "/blabla",
    "vol_snap_path": "/blablasnap",
    "total_broken_links": 0,
    "total_fs_files": 11,
    "total_directories": 2,
    "backup_size_uncompressed": 4567,
    "backup_size_compressed": 1212,
    "total_backup_session_size": 6789,
    "compression_alg": "None",
    "encrypted": "false",
    "client_os": "linux",
    "broken_links": ["link_01", "link_02"],
    "excluded_files": ["excluded_file_01", "excluded_file_02"],
    "cli": ""
}

fake_data_0_elasticsearch_hit = {
    "_shards": {
        "failed": 0,
        "successful": 5,
        "total": 5
    },
    "hits": {
        "hits": [
            {
                "_id": "AUx_iu-ewlhuOVELWtH0",
                "_index": "freezer",
                "_score": 1.0,
                "_type": "backups",
                "_source": {
                    "container": "freezer_container",
                    "hostname": "alpha",
                    "backup_name": "important_data_backup",
                    "time_stamp": 8475903425,
                    "curr_backup_level": 0,
                    "backup_session": 8475903425,
                    "max_level": 5,
                    "mode": "fs",
                    "fs_real_path": "/blabla",
                    "vol_snap_path": "/blablasnap",
                    "total_broken_links": 0,
                    "total_fs_files": 11,
                    "total_directories": 2,
                    "backup_size_uncompressed": 4567,
                    "backup_size_compressed": 1212,
                    "total_backup_session_size": 6789,
                    "compression_alg": "None",
                    "encrypted": "false",
                    "client_os": "linux",
                    "broken_links": ["link_01", "link_02"],
                    "excluded_files": ["excluded_file_01", "excluded_file_02"],
                    "cli": ""
                }
            }
        ],
        "max_score": 1.0,
        "total": 1
    },
    "timed_out": False,
    "took": 3
}

fake_data_0_elasticsearch_miss = {
    "_shards": {
        "failed": 0,
        "successful": 5,
        "total": 5
    },
    "hits": {
        "hits": [],
        "max_score": None,
        "total": 0
    },
    "timed_out": False,
    "took": 1
}

fake_job_0_project_id = "tecs"
fake_job_0_user_id = "f4db4da085f043059441565720b217c7"
fake_job_0_job_id = "e7181e5e-2c75-43f8-92c0-c037ae5f11e4"

fake_job_0_elasticsearch_not_found = {
    "_id": "e7181e5e-2c75-43f8-92c0-c037ae5f11e43",
    "_index": "freezer",
    "_type": "job",
    "found": False
}

fake_job_0 = {
    "job_actions": [
        {
            "freezer_action": {
                "action": "backup",
                "mode": "fs",
                "path_to_backup": "/home/tylerdurden/project_mayhem",
                "backup_name": "project_mayhem_backup",
                "container": "my_backup_container"
            },
            "max_retries": 3,
            "max_retries_interval": 60,
            "mandatory": False
        },
        {
            "freezer_action": {
                "action": "restore",
                "mode": "fs",
                "restore_abs_path": "/home/tylerdurden/project_mayhem",
                "restore_from_host": "node_on_which_backup_was_made",
                "backup_name": "project_mayhem_backup",
                "container": "my_backup_container"
            },
            "max_retries": 3,
            "max_retries_interval": 60,
            "mandatory": True
        }
    ],
    "job_schedule": {
        "time_created": 1234,
        "time_started": 1234,
        "time_ended": 1234,
        "status": "stop",
        "result": "success",
        "schedule_date": "2015-06-02T16:20:00",
        "schedule_interval": "2 days"
    },
    "job_id": "e7181e5e-2c75-43f8-92c0-c037ae5f11e4",
    "client_id": "mytenantid_myhostname",
    "user_id": "f4db4da085f043059441565720b217c7",
    "project_id": "tecs",
    "description": "test action 4"
}


fake_job_2 = {
    "job_actions": [
        {
            "freezer_action": {
                "action": "backup",
                "mode": "fs",
                "path_to_backup": "/home/tylerdurden/project_mayhem",
                "backup_name": "project_mayhem_backup",
                "container": "my_backup_container1"
            },
            "max_retries": 6,
            "max_retries_interval": 100,
            "mandatory": False
        },
        {
            "freezer_action": {
                "action": "restore",
                "mode": "fs",
                "restore_abs_path": "/home/tylerdurden/project_mayhem",
                "restore_from_host": "node_on_which_backup_was_made",
                "backup_name": "project_mayhem_backup",
                "container": "my_backup_container1"
            },
            "max_retries": 4,
            "max_retries_interval": 60,
            "mandatory": True
        }
    ],
    "job_schedule": {
        "time_created": 1234,
        "time_started": 1234,
        "time_ended": 1234,
        "status": "stop",
        "result": "success",
        "schedule_date": "2015-06-02T16:20:00",
        "schedule_interval": "7 days"
    },
    "job_id": "e7181e5e-2c75-43f8-92c0-c037ae5f11e4",
    "client_id": "mytenantid_myhostname1",
    "user_id": "f4db4da085f043059441565720b217c7",
    "project_id": "tecs",
    "description": "test action 5"
}

fake_job_3 = {
    "job_actions": [
        {
            "freezer_action": {
                "action": "backup",
                "mode": "fs",
                "path_to_backup": "/home/tylerdurden/project_mayhem",
                "backup_name": "project_mayhem_backup",
                "container": "my_backup_container1"
            },
            "max_retries": 3,
            "max_retries_interval": 150,
            "mandatory": False
        },
        {
            "freezer_action": {
                "action": "restore",
                "mode": "fs",
                "restore_abs_path": "/home/tylerdurden/project_mayhem",
                "restore_from_host": "node_on_which_backup_was_made",
                "backup_name": "project_mayhem_backup",
                "container": "my_backup_container1"
            },
            "max_retries": 4,
            "max_retries_interval": 60,
            "mandatory": True
        }
    ],
    "job_schedule": {
        "time_created": 1234,
        "time_started": 1234,
        "time_ended": 1234,
        "status": "stop",
        "result": "success",
        "schedule_date": "2015-06-02T16:20:00",
        "schedule_interval": "14 days"
    },
    "job_id": "e7181e5e-2c75-43f8-92c0-c037ae5f11e5",
    "client_id": "mytenantid_myhostname2",
    "user_id": "f4db4da085f043059441565720b217c7",
    "project_id": "tecs",
    "description": "test action 6"
}


def get_fake_job_0():
    return copy.deepcopy(fake_job_0)


def get_fake_job_1():
    job = copy.deepcopy(fake_job_0)
    job["job_id"] = 'pqoqurioew'
    return job


def get_fake_job_2():
    return copy.deepcopy(fake_job_2)


def get_fake_job_3():
    return copy.deepcopy(fake_job_3)


def get_fake_job_id():
    return uuid.uuid4().hex


fake_job_0_elasticsearch_found = {
    "_id": "e7181e5e-2c75-43f8-92c0-c037ae5f11e4",
    "_index": "freezer",
    "_source": fake_job_0,
    "_type": "actions",
    "_version": 1,
    "found": True
}

fake_data_1_wrapped_backup_metadata = {
    'backup_id': 'b740ed9ad2b646aba304ef54c21c6774',
    'user_id': 'qwerty1234',
    'user_name': 'asdffdsa',
    'backup_metadata': {
        "container": "freezer_container",
        "hostname": "alpha",
        "backup_name": "important_data_backup",
        "time_stamp": 125235431,
        "curr_backup_level": 1,
        "backup_session": 8475903425,
        "max_level": 5,
        "mode": "fs",
        "fs_real_path": "/blabla",
        "vol_snap_path": "/blablasnap",
        "total_broken_links": 0,
        "total_fs_files": 11,
        "total_directories": 2,
        "backup_size_uncompressed": 4567,
        "backup_size_compressed": 1212,
        "total_backup_session_size": 6789,
        "compression_alg": "None",
        "encrypted": "false",
        "client_os": "linux",
        "broken_links": ["link_01", "link_02"],
        "excluded_files": ["excluded_file_01", "excluded_file_02"],
        "cli": ""
    }
}

fake_client_info_0 = {
    "project_id": "tecs",
    "client_id": "test-tenant_5253_test-hostname_09544",
    "description": "some usefule text here",
    "config_id": "config_id_contains_uuid_of_config"
}

fake_client_info_1 = {
    "project_id": "tecs",
    "client_id": "test-tenant_5253_test-hostname_6543",
    "description": "also some useful text here",
    "config_id": "config_id_blablawhatever"
}

fake_client_entry_0 = {
    "client": fake_client_info_0,
    "user_id": "user_id-is-provided-keystone"
}

fake_client_entry_1 = {
    "client": fake_client_info_0,
    "user_id": "user_id-is-provided-keystone"
}


def get_fake_client_0():
    return copy.deepcopy(fake_client_entry_0)


def get_fake_client_1():
    return copy.deepcopy(fake_client_entry_1)


def get_fake_client_id():
    return uuid.uuid4().hex


fake_action_0 = {
    "freezer_action":
        {
            "action": "backup",
            "mode": "fs",
            "path_to_backup": "/home/tylerdurden/project_mayhem",
            "backup_name": "project_mayhem_backup",
            "container": "my_backup_container",
        },
    "exit_status": "success",
    "max_retries": 3,
    "max_retries_interval": 60,
    "mandatory": True,

    "action_id": "qwerqwerqwerrewq",
    "user_id": "user_id-is-provided-by-keystone",
    "project_id": "project_id-is-tecs"
}

fake_action_1 = {
    "freezer_action":
        {
            "action": "backup",
            "mode": "fs",
            "path_to_backup": "/home/tylerdurden/project_mayhem",
            "backup_name": "project_mayhem_backup",
            "container": "my_backup_container",
        },
    "exit_status": "success",
    "max_retries": 3,
    "max_retries_interval": 60,
    "mandatory": True,

    "action_id": "jk4lkjbn4r3k",
    "user_id": "user_id-is-provided-by-keystone",
    "project_id": "project_id-is-tecs"
}

fake_action_2 = {
    "freezer_action":
        {
            "action": "restore",
            "mode": "fs",
            "path_to_backup": "/home/tecs/project_tecs",
            "backup_name": "project_tecs_restore",
            "container": "my_restore_container",
        },
    "exit_status": "success",
    "max_retries": 5,
    "max_retries_interval": 70,
    "mandatory": False,
    "action_id": "qwerqwerqwerrewq",
    "user_id": "user_id-is-provided-by-keystone",
    "project_id": "project_id-is-tecs"
}

fake_action_3 = {
    "freezer_action":
        {
            "action": "restore",
            "mode": "fs",
            "path_to_backup": "/home/tecs/project_tecs",
            "backup_name": "project_tecs_restore",
            "container": "my_restore_container",
        },
    "exit_status": "success",
    "max_retries": 5,
    "max_retries_interval": 70,
    "mandatory": False,
    "user_id": "user_id-is-provided-by-keystone",
    "project_id": "project_id-is-tecs"
}


def get_fake_action_0():
    return copy.deepcopy(fake_action_0)


def get_fake_action_1():
    return copy.deepcopy(fake_action_1)


def get_fake_action_2():
    return copy.deepcopy(fake_action_2)


def get_fake_action_id():
    return uuid.uuid4().hex


def get_fake_action_3():
    return copy.deepcopy(fake_action_3)


fake_session_0 = {
    "session_id": 'turistidellademocrazia',
    "session_tag": 5,
    "description": 'some text here',
    "hold_off": 60,
    "schedule": {
        "time_created": 1234,
        "time_started": 1234,
        "time_ended": 0,
        "status": "stop",
        "schedule_date": "2015-06-02T16:20:00"
    },
    "jobs": {
        'venerescollataincorpodalolita': {
            "client_id": 'bruco',
            "status": 'running',
            "start_time": 12344321,
        },
        'job_id_2': {
            "client_id": "cocktail",
            "status": 'completed',
            "result": 'success',
            "start_time": 123321,
            "end_time": 123325,
        }
    },
    "time_start": 123412344,
    "time_end": 432234432,
    "status": "running",
    "user_id": "califfo",
    "project_id": "tecs"
}

fake_session_1 = {
    "session_id": 'turistidellademocrazia',
    "session_tag": 5,
    "description": 'some text here',
    "hold_off": 60,
    "schedule": {
        "time_created": 1234,
        "time_started": 1234,
        "time_ended": 0,
        "status": "stop",
        "schedule_date": "2015-06-02T16:20:00"
    },
    "jobs": {
        'venerescollataincorpodalolita': {
            "client_id": 'bruco',
            "status": 'running',
            "start_time": 12344321,
        }
    },
    "time_start": 123412344,
    "time_end": 432234432,
    "status": "running",
    "user_id": "califfo",
    "project_id": "tecs"
}

fake_session_2 = {
    "session_id": 'turistidellademocrazia',
    "session_tag": 5,
    "description": 'This is a session about test',
    "hold_off": 100,
    "schedule": {
        "time_created": 1234,
        "time_started": 1234,
        "time_ended": 0,
        "status": "stop",
        "schedule_date": "2018-11-14T16:20:00"
    },
    "jobs": {
        'venerescollataincorpodalolita': {
            "client_id": 'bruco',
            "status": 'running',
            "start_time": 12344321,
        },
        'job_id_2': {
            "client_id": "cocktail",
            "status": 'completed',
            "result": 'success',
            "start_time": 123321,
            "end_time": 123325,
        }
    },
    "time_start": 123412344,
    "time_end": 432234432,
    "status": "running",
    "user_id": "califfo",
    "project_id": "tecs"
}

fake_session_3 = {
    "session_id": 'turistidellademocrazia',
    "session_tag": 6,
    "description": 'This is a session about test',
    "hold_off": 150,
    "schedule": {
        "time_created": 1234,
        "time_started": 1234,
        "time_ended": 0,
        "status": "stop",
        "schedule_date": "2018-11-14T16:20:00"
    },
    "jobs": {
        'venerescollataincorpodalolita': {
            "client_id": 'bruco',
            "status": 'running',
            "start_time": 12344321,
        },
        'job_id_2': {
            "client_id": "cocktail",
            "status": 'completed',
            "result": 'success',
            "start_time": 123321,
            "end_time": 123325,
        }
    },
    "time_start": 123412344,
    "time_end": 432234432,
    "status": "running",
    "user_id": "califfo",
    "project_id": "tecs"
}


def get_fake_session_0():
    return copy.deepcopy(fake_session_0)


def get_fake_session_1():
    return copy.deepcopy(fake_session_1)


def get_fake_session_2():
    return copy.deepcopy(fake_session_2)


def get_fake_session_3():
    return copy.deepcopy(fake_session_3)


def get_fake_session_id():
    return uuid.uuid4().hex


class FakeReqResp(object):
    def __init__(self, method='GET', body=''):
        self.method = method
        self.body = body
        self.stream = io.BytesIO(body)
        self.content_length = len(body)
        self.context = {}
        self.header = {}

    def get_header(self, key):
        return self.header.get(key, None)


class FreezerBaseTestCase(testtools.TestCase):
    def setUp(self):
        super(FreezerBaseTestCase, self).setUp()

        self._config_fixture = self.useFixture(cfg_fixture.Config())
        config.parse_args(args=[])
        self.addCleanup(CONF.reset)
        self.test_dir = self.useFixture(fixtures.TempDir()).path
        self.conf_dir = os.path.join(self.test_dir, 'etc')
        os.makedirs(self.conf_dir)
        policy.ENFORCER = None
        policy.setup_policy(CONF)


class FakeContext(object):
    def __init__(self, *args, **kwargs):
        self.context = {}

    def to_dict(self):
        return self.context


def get_req_items(name):
    req_info = {'freezer.context': FakeContext()}
    return req_info[name]



# -*- encoding: utf-8 -*-
# (c) Copyright 2018 ZTE Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""Tests for manipulating job via the DB API"""

import copy
from unittest import mock
from unittest.mock import patch

from oslo_config import cfg

from freezer_api.common import exceptions as freezer_api_exc
from freezer_api.db.sqlalchemy import api
from freezer_api.db.sqlalchemy import models
from freezer_api.tests.unit import common
from freezer_api.tests.unit.sqlalchemy import base

CONF = cfg.CONF


class ApiTestCase(base.DbTestCase):

    def setUp(self):
        super(ApiTestCase, self).setUp()
        self.fake_job_0 = common.get_fake_job_0()
        self.fake_client_0 = common.get_fake_client_0()
        self.fake_user_id = self.fake_job_0.get('user_id')
        self.fake_job_id = common.get_fake_job_id()
        self.fake_action_0 = common.get_fake_action_0()
        self.fake_backup_0 = common.get_fake_backup_metadata()
        self.fake_session_0 = common.get_fake_session_0()
        self.fake_session_id = common.get_fake_session_id()
        CONF.enable_v1_api = True

    def tearDown(self):
        super(ApiTestCase, self).tearDown()
        CONF.enable_v1_api = False

    def test_raises_model_query(self):
        session = api.get_db_session()
        session.begin()
        self.assertRaises(ValueError,
                          api.model_query, session, models.Job,
                          read_deleted='both')
        session.close()

    def test_model_query(self):
        job_doc1 = copy.deepcopy(self.fake_job_0)
        job_doc2 = copy.deepcopy(self.fake_job_0)
        self.dbapi.add_job(user_id=self.fake_user_id,
                           doc=job_doc1)
        job_id2 = self.dbapi.add_job(user_id=self.fake_user_id,
                                     doc=job_doc2)

        result = self.dbapi.delete_job(user_id=self.fake_user_id,
                                       job_id=job_id2)
        session = api.get_db_session()
        with session.begin():
            try:
                query = api.model_query(session, models.Job,
                                        read_deleted='no')
                query = query.filter_by(user_id=self.fake_user_id)
                result = query.all()
                self.assertEqual(len(result), 1)

                query = api.model_query(session, models.Job,
                                        read_deleted='only')
                query = query.filter_by(user_id=self.fake_user_id)
                result = query.all()
                self.assertEqual(len(result), 1)

                query = api.model_query(session, models.Job,
                                        read_deleted='yes')
                query = query.filter_by(user_id=self.fake_user_id)
                result = query.all()
                self.assertEqual(len(result), 2)
            except Exception as e:
                raise freezer_api_exc.StorageEngineError(
                    message='sqlalchemy operation failed {0}'.format(e))
        session.close()

    @ patch('oslo_db.sqlalchemy.utils.model_query')
    def test_raises_delete_tuple(self, mock_model_query):
        mock_model_query.side_effect = Exception('regular test failure')
        self.assertRaises(freezer_api_exc.StorageEngineError,
                          api.delete_tuple, models.Job, self.fake_user_id,
                          self.fake_job_id)

    def test_delete_tuple(self):
        job_doc1 = copy.deepcopy(self.fake_job_0)
        job_doc2 = copy.deepcopy(self.fake_job_0)
        self.dbapi.add_job(user_id=self.fake_user_id,
                           doc=job_doc1)
        job_id2 = self.dbapi.add_job(user_id=self.fake_user_id,
                                     doc=job_doc2)

        result = self.dbapi.delete_job(user_id=self.fake_user_id,
                                       job_id=job_id2)
        session = api.get_db_session()
        with session.begin():
            try:
                query = api.model_query(session, models.Job,
                                        read_deleted='no')
                query = query.filter_by(user_id=self.fake_user_id)
                result = query.all()
                self.assertEqual(len(result), 1)

                query = api.model_query(session, models.Job,
                                        read_deleted='only')
                query = query.filter_by(user_id=self.fake_user_id)
                result = query.all()
                self.assertEqual(len(result), 1)

                query = api.model_query(session, models.Job,
                                        read_deleted='yes')
                query = query.filter_by(user_id=self.fake_user_id)
                result = query.all()
                self.assertEqual(len(result), 2)
            except Exception as e:
                raise freezer_api_exc.StorageEngineError(
                    message='sqlalchemy operation failed {0}'.format(e))
        session.close()

    @patch('oslo_db.sqlalchemy.utils.model_query')
    def test_raises_get_tuple(self, mock_model_query):
        mock_model_query.side_effect = Exception('regular test failure')
        self.assertRaises(freezer_api_exc.StorageEngineError,
                          api.get_tuple, models.Job, self.fake_user_id,
                          self.fake_job_id)

    def test_raises_add_tuple(self):
        mock_tuple = mock.MagicMock()
        mock_tuple.save.side_effect = Exception('regular test failure')
        self.assertRaises(freezer_api_exc.StorageEngineError,
                          api.add_tuple, mock_tuple)

    @patch('oslo_db.sqlalchemy.utils.model_query')
    def test_raises_update_tuple(self, mock_model_query):
        mock_tuple_values = mock.MagicMock()
        mock_model_query.side_effect = Exception('regular test failure')
        self.assertRaises(freezer_api_exc.StorageEngineError,
                          api.update_tuple, models.Job, self.fake_user_id,
                          self.fake_job_id, mock_tuple_values)

    def test_raises_update_tuple_notfound(self):
        mock_tuple_values = mock.MagicMock()
        self.assertRaises(freezer_api_exc.DocumentNotFound,
                          api.update_tuple, models.Job, self.fake_user_id,
                          self.fake_job_id, mock_tuple_values)

    @patch('oslo_db.sqlalchemy.utils.model_query')
    def test_raises_replace_tuple(self, mock_model_query):
        mock_tuple_values = mock.MagicMock()
        mock_model_query.side_effect = Exception('regular test failure')
        self.assertRaises(freezer_api_exc.StorageEngineError,
                          api.replace_tuple, models.Job, self.fake_user_id,
                          self.fake_job_id, mock_tuple_values)

    @patch('oslo_db.sqlalchemy.utils.model_query')
    def test_raises_search_tuple(self, mock_model_query):
        mock_tablename = mock.MagicMock()
        mock_tablename = mock.MagicMock()
        mock_search = mock.MagicMock()
        mock_model_query.side_effect = Exception('regular test failure')
        self.assertRaises(freezer_api_exc.StorageEngineError,
                          api.search_tuple, mock_tablename, self.fake_user_id,
                          offset=0, limit=100,
                          search=mock_search)

    def test_valid_and_get_search_option(self):
        search = {'error_key': 'search_info_error'}
        result = api.valid_and_get_search_option(search)
        self.assertEqual({}, result)

    def test_get_recursively_list_include_dict(self):
        dict1 = {'key1': [{'key11': {'key111': 111}},
                          {'key12': {'key121': 121}}],
                 'key2': 2
                 }
        search_key = {'key111': 111,
                      'key121': 121,
                      'key2': 2}
        search_keys_found = api.get_recursively(dict1, search_key)
        self.assertEqual(search_key, search_keys_found)



"""
(c) Copyright 2015-2016 Hewlett-Packard Enterprise Development Company, L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""
import sys

from oslo_config import cfg
from oslo_log import log
from oslo_serialization import jsonutils as json

from freezer_api import __version__ as FREEZER_API_VERSION
from freezer_api.common import config
from freezer_api.db import manager


CONF = cfg.CONF
LOG = log.getLogger(__name__)


DEFAULT_INDEX = 'freezer'
DEFAULT_REPLICAS = 0


def add_db_opts(subparser):
    parser = subparser.add_parser('db')
    parser.add_argument(
        'options',
        choices=['sync', 'update', 'remove', 'show', 'update-settings'],
        help='Create/update/delete freezer-api mappings in DB backend.'
    )


def parse_config():
    DB_INIT = [
        cfg.SubCommandOpt('db',
                          dest='db',
                          title='DB Options',
                          handler=add_db_opts
                          )
    ]
    # register database backend drivers
    config.register_db_drivers_opt()
    # register database cli options
    CONF.register_cli_opts(DB_INIT)
    # register logging opts
    log.register_options(CONF)
    default_config_files = cfg.find_config_files('freezer', 'freezer-api')
    CONF(args=sys.argv[1:],
         project='freezer-api',
         default_config_files=default_config_files,
         version=FREEZER_API_VERSION
         )


def main():
    parse_config()
    config.setup_logging()

    if not CONF.db:
        CONF.print_help()
        sys.exit(0)

    try:
        db_driver = manager.get_db_driver(CONF.storage.driver,
                                          backend=CONF.storage.backend)
        if CONF.db.options.lower() == 'sync':
            db_driver.db_sync()
        elif CONF.db.options.lower() == 'update':
            db_driver.db_sync()
        elif CONF.db.options.lower() == 'remove':
            db_driver.db_remove()
        elif CONF.db.options.lower() == 'show':
            db_tables = db_driver.db_show()
            if db_tables:
                print(json.dumps(db_tables))
            else:
                print("No Tables/Mappings found!")
        else:
            raise Exception('Option {0} not found !'.format(CONF.db.options))
    except Exception as e:
        LOG.error(e)
        print(e)


if __name__ == '__main__':
    sys.exit(main())



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from oslo_config import cfg
from oslo_log import log

from freezer_api.common import db_mappings
from freezer_api.db import base as db_base
from freezer_api.db.elasticsearch import es_manager
from freezer_api.storage import elastic as db_session_v1
from freezer_api.storage import elasticv2 as db_session_v2

CONF = cfg.CONF
LOG = log.getLogger(__name__)
DEFAULT_INDEX = 'freezer'
DEFAULT_REPLICAS = 0

_BACKEND_MAPPING = {'sqlalchemy': 'freezer_api.db.sqlalchemy.api'}


class ElasticSearchDB(db_base.DBDriver):
    _ES_OPTS = [
        cfg.ListOpt('hosts',
                    default=['http://127.0.0.1:9200'],
                    help='specify the storage hosts'),
        cfg.StrOpt('index',
                   default='freezer',
                   help='specify the name of the elasticsearch index'),
        cfg.IntOpt('timeout',
                   default=60,
                   help='specify the connection timeout'),
        cfg.IntOpt('retries',
                   default=20,
                   help='number of retries to allow before raising and error'),
        cfg.BoolOpt('use_ssl',
                    default=False,
                    help='explicitly turn on SSL'),
        cfg.BoolOpt('verify_certs',
                    default=False,
                    help='turn on SSL certs verification'),
        cfg.StrOpt('ca_certs',
                   help='path to CA certs on disk'),
        cfg.IntOpt('number_of_replicas',
                   default=0,
                   help='Number of replicas for elk cluster. Default is 0. '
                        'Use 0 for no replicas. This should be set to (number '
                        'of node in the ES cluter -1).'),
        cfg.StrOpt('mapping',
                   dest='select_mapping',
                   default='',
                   help='Specific mapping to upload. Valid choices: {0}'
                   .format(','.join(db_mappings.get_mappings()))),
        cfg.BoolOpt('erase',
                    dest='erase',
                    default=False,
                    help='Enable index deletion in case mapping update fails '
                         'due to incompatible changes'
                    ),
        cfg.StrOpt('test-only',
                   dest='test_only',
                   default=False,
                   help='Test the validity of the mappings, but take no action'
                   )
    ]

    def __init__(self, backend):
        super(ElasticSearchDB, self).__init__(backend)
        grp = cfg.OptGroup(backend)
        CONF.register_group(grp)
        CONF.register_opts(self._ES_OPTS, group=backend)
        # CONF.register_cli_opts(self._ES_CLI_OPTS)

        self.conf = CONF.get(backend)
        self.index = self.conf.index or DEFAULT_INDEX
        self._engine = None
        self._manage_engine = None

    def get_engine(self):
        if not self._engine:
            if CONF.enable_v1_api:
                self._engine = db_session_v1.\
                    ElasticSearchEngine(self.backend)
            else:
                self._engine = db_session_v2.\
                    ElasticSearchEngineV2(self.backend)
        return self._engine

    def get_api(self):
        return self.get_engine()

    def get_manage_engine(self):
        opts = dict(self.conf.items())
        self._manage_engine = es_manager.ElasticSearchManager(**opts)
        return self._manage_engine

    def db_sync(self):
        if not self._manage_engine:
            self._manage_engine = self.get_manage_engine()
        self._manage_engine.update_mappings()

    def db_remove(self):
        if not self._manage_engine:
            self._manage_engine = self.get_manage_engine()
        self._manage_engine.remove_mappings()

    def db_show(self):
        if not self._manage_engine:
            self._manage_engine = self.get_manage_engine()
        return self._manage_engine.show_mappings()

    def name(self):
        return "ElasticSearch"



"""
Copyright 2015 Hewlett-Packard

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

import random

import falcon
from unittest import mock

from freezer_api.api.v2 import actions as v2_actions
from freezer_api.common import exceptions
from freezer_api.tests.unit import common


class TestActionsCollectionResource(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestActionsCollectionResource, self).setUp()
        self.mock_db = mock.Mock()
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.get_header.return_value = common.fake_action_0['user_id']
        self.mock_req.status = falcon.HTTP_200
        self.resource = v2_actions.ActionsCollectionResource(self.mock_db)
        self.mock_json_body = mock.Mock()
        self.mock_json_body.return_value = {}
        self.resource.json_body = self.mock_json_body

    def test_on_get_return_empty_list(self):
        self.mock_db.search_action.return_value = []
        expected_result = {'actions': []}
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_action_0['project_id'])
        result = self.mock_req.body
        self.assertEqual(expected_result, result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_get_return_correct_list(self):
        self.mock_db.search_action.return_value = [common.get_fake_action_0(),
                                                   common.get_fake_action_1()]
        expected_result = {'actions': [common.get_fake_action_0(),
                                       common.get_fake_action_1()]}
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_action_0['project_id'])
        result = self.mock_req.body
        self.assertEqual(expected_result, result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_post_raises_when_missing_body(self):
        self.mock_db.add_action.return_value = common.fake_action_0[
            'action_id']
        self.assertRaises(exceptions.BadDataFormat, self.resource.on_post,
                          self.mock_req, self.mock_req, self.mock_req)

    def test_on_post_inserts_correct_data(self):
        action = common.get_fake_action_0()
        self.mock_json_body.return_value = action
        self.mock_db.add_action.return_value = 'pjiofrdslaikfunr'
        expected_result = {'action_id': 'pjiofrdslaikfunr'}
        self.resource.on_post(self.mock_req, self.mock_req,
                              common.fake_action_0['project_id'])
        self.assertEqual(falcon.HTTP_201, self.mock_req.status)
        self.assertEqual(expected_result, self.mock_req.body)


class TestActionsResource(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestActionsResource, self).setUp()
        self.mock_db = mock.Mock()
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.get_header.return_value = common.fake_action_0['user_id']
        self.mock_req.status = falcon.HTTP_200
        self.resource = v2_actions.ActionsResource(self.mock_db)
        self.mock_json_body = mock.Mock()
        self.mock_json_body.return_value = {}
        self.resource.json_body = self.mock_json_body

    def test_create_resource(self):
        self.assertIsInstance(self.resource, v2_actions.ActionsResource)

    def test_on_get_return_no_result_and_404_when_not_found(self):
        self.mock_db.get_action.return_value = None
        self.mock_req.body = None
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_action_0['project_id'],
                             common.fake_action_0['action_id'])
        self.assertIsNone(self.mock_req.body)
        self.assertEqual(falcon.HTTP_404, self.mock_req.status)

    def test_on_get_return_correct_data(self):
        self.mock_db.get_action.return_value = common.get_fake_action_0()
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_action_0['project_id'],
                             common.fake_action_0['action_id'])
        result = self.mock_req.body
        self.assertEqual(common.get_fake_action_0(), result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_delete_removes_proper_data(self):
        self.resource.on_delete(self.mock_req, self.mock_req,
                                common.fake_action_0['project_id'],
                                common.fake_action_0['action_id'])
        result = self.mock_req.body
        expected_result = {'action_id': common.fake_action_0['action_id']}
        self.assertEqual(falcon.HTTP_204, self.mock_req.status)
        self.assertEqual(expected_result, result)

    def test_on_patch_ok_with_some_fields(self):
        new_version = random.randint(0, 99)
        self.mock_db.update_action.return_value = new_version
        patch_doc = {'some_field': 'some_value',
                     'because': 'size_matters'}
        self.mock_json_body.return_value = patch_doc

        patch_doc.copy()

        expected_result = {'action_id': common.fake_action_0['action_id'],
                           'version': new_version}

        self.resource.on_patch(self.mock_req, self.mock_req,
                               common.fake_action_0['project_id'],
                               common.fake_action_0['action_id'])
        self.mock_db.update_action.assert_called_with(
            user_id=common.fake_action_0['user_id'],
            project_id=common.fake_action_0['project_id'],
            action_id=common.fake_action_0['action_id'],
            patch_doc=patch_doc)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)
        result = self.mock_req.body
        self.assertEqual(expected_result, result)

    def test_on_post_ok(self):
        new_version = random.randint(0, 99)
        self.mock_db.replace_action.return_value = new_version
        action = common.get_fake_action_0()
        self.mock_json_body.return_value = action
        expected_result = {'action_id': common.fake_action_0['action_id'],
                           'version': new_version}

        self.resource.on_post(self.mock_req, self.mock_req,
                              common.fake_action_0['project_id'],
                              common.fake_action_0['action_id'])
        self.assertEqual(falcon.HTTP_201, self.mock_req.status)
        self.assertEqual(expected_result, self.mock_req.body)

    def test_on_post_raises_when_db_replace_action_raises(self):
        self.mock_db.replace_action.side_effect = exceptions.AccessForbidden(
            'regular test failure')
        action = common.get_fake_action_0()
        self.mock_json_body.return_value = action
        self.assertRaises(exceptions.AccessForbidden, self.resource.on_post,
                          self.mock_req,
                          self.mock_req,
                          common.fake_action_0['project_id'],
                          common.fake_action_0['action_id'])



"""
(c) Copyright 2016 Hewlett-Packard Enterprise Development Company, L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""
import sys

from oslo_config import cfg
from paste import deploy
from paste import urlmap

from freezer_api.cmd import api
from freezer_api.common import config

CONF = cfg.CONF


def root_app_factory(loader, global_conf, **local_conf):
    """Freezer can manage multiple versions, but
    only launch one version at a time, Otherwise it is
     easy to cause confusion. If there is a demand in the future
    for a single freezer-api instance to support both v1 and v2 at a time,
    you need to add a new patch to implement it.
    """
    if CONF.enable_v1_api and '/v1' in local_conf:
        del local_conf['/v2']
    else:
        del local_conf['/v1']
    return urlmap.urlmap_factory(loader, global_conf, **local_conf)


def freezer_appv1_factory(global_conf, **local_conf):
    return api.build_app_v1()


def freezer_appv2_factory(global_conf, **local_conf):
    return api.build_app_v2()


def initialize_app(conf=None, name='main'):
    """ initializing app for paste to deploy it """

    # register and parse arguments
    config.parse_args(args=sys.argv[1:])
    # register logging opts
    config.setup_logging()
    # locate and load paste file
    conf = config.find_paste_config()
    app = deploy.loadapp('config:%s' % conf, name=name)
    return app



# Copyright 2015 Hewlett-Packard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Base resource with utility functions

"""

import falcon
from oslo_serialization import jsonutils as json

from freezer_api.common import exceptions as freezer_api_exc


class BaseResource(object):

    @staticmethod
    def json_body(req):
        if not req.content_length:
            return {}

        try:
            raw_json = req.stream.read()
        except Exception:
            raise freezer_api_exc.BadDataFormat('Empty request body. A valid '
                                                'JSON document is required.')
        try:
            json_data = json.loads(raw_json, encoding='utf-8')
        except ValueError:
            raise falcon.HTTPError(falcon.HTTP_753,
                                   'Malformed JSON')
        return json_data
